package org.zxq.teleri.account

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.SpannableString
import android.text.Spanned
import android.text.TextPaint
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import com.alibaba.android.arouter.launcher.ARouter
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.zxq.teleri.BrandHelper
import org.zxq.teleri.R
import org.zxq.teleri.account.datamodel.UserLogin
import org.zxq.teleri.arouter.constant.ARouterUrl
import org.zxq.teleri.bean.CheckCancellationAccountBean
import org.zxq.teleri.core.ex.OnErrorResponse
import org.zxq.teleri.core.net.exception.ErrorResponseException
import org.zxq.teleri.core.net.response.ErrorResponseA
import org.zxq.teleri.core.utils.Json
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.dialog.*
import org.zxq.teleri.model.request.open.CancellationAccountRequest
import org.zxq.teleri.model.request.open.CheckCancellationAccountRequest
import org.zxq.teleri.personalcenter.PCHelper
import org.zxq.teleri.ui.base.SimpleBaseActivity
import org.zxq.teleri.ui.utils.AppUtils
import org.zxq.teleri.ui.web.BaseWebActivity
import org.zxq.teleri.ui.windvareweb.BasicWebData
import org.zxq.teleri.utils.StringUtils
import org.zxq.teleri.utils.ZXQUtils
import org.zxq.teleri.web.InnerWebViewActivity

/**
 * Created by leixiong on 2018/12/17.
 */
class AccountCancellationActivity : SimpleBaseActivity(), View.OnClickListener {

    val CARNOTUBOUNDCODE = "41801"//有车辆未解绑
    val UNFULFILLEDRDER = "41802"//有未完成的订单
    val CHECKBALANCE = "41803"//有未提现的车钱包余额
    val CHECKSUCCESS = "0"//条件满足 可以进行注销
    lateinit var btCancellation: Button
    lateinit var tvAncellationContent5: TextView
    lateinit var tvAncellationContent6: TextView
    lateinit var tvMobile: TextView
    var normalDialog: NormalDialog? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initView()
    }

    fun initView() {
        setContentView(R.layout.activity_account_cancellation)
        setTitle(R.string.account_cancellation)
        btCancellation = findViewById(R.id.bt_cancellation)
        tvMobile = findViewById(R.id.tv_mobile)
        tvAncellationContent5 = findViewById(R.id.tv_ancellation_content5)
        tvAncellationContent6 = findViewById(R.id.tv_ancellation_content6)
        btCancellation.setOnClickListener(this)
        findViewById<ImageView>(R.id.back).setOnClickListener(this)
    }

    override fun onClick(v: View?) {
        super.onClick(v)
        if (AppUtils.isFastDoubleClick()) return
        when (v?.id) {
            R.id.bt_cancellation -> {
                val normalDialog = NormalDialog(this@AccountCancellationActivity, NormalDialog.VERTICAL_TWO,
                        arrayOf(getString(R.string.cancellation), getString(R.string.cancellation_cancel)))
                normalDialog?.setUseStyle(false, "#FF0000", "#FF8F8E94", null)
                normalDialog.setDialogContent(getString(R.string.cancellation_tips))
                normalDialog.tvDialogTitle.visibility = View.GONE
                normalDialog.setClickButtonListener(object : VerticalTwoButtonClick(normalDialog) {
                    override fun onTopClick() {
                        cancellation()
                    }

                    override fun onBottomClick() {

                    }
                })
                normalDialog.show()
            }
        }
    }

    fun cancellation() {
        showLoadingDialog()
        //closeUserAccount接口 包含 check接口的功能 ,所以只需要用closeUserAccount接口就OK..
        var checkDisposable = CancellationAccountRequest().prepare()
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ result ->

                    var closeCancellationAccountBean = Json.from(result, CheckCancellationAccountBean::class.java)
                    //closeCancellationAccountBean.code=CHECKBALANCE
                    checkAccountCancellactionHandler(closeCancellationAccountBean)


                }, {
                    closeLoadingDialog()
                    OnErrorResponse.getInstance().accept(it)

                })

        mCompositeDisposable.add(checkDisposable)

    }

    fun checkAccountCancellactionHandler(checkCancellationAccountBean: CheckCancellationAccountBean): Boolean {
        var isCheckSuccess = false
        var code = checkCancellationAccountBean.code
        closeLoadingDialog()
        when (code) {
            //注销成功
            CHECKSUCCESS -> {
                AppUtils.showToast(R.string.account_cancellation_success)
                ZXQUtils.deleteUserMobile(UserLogin.getAccountA().mobile)
                ZXQUtils.gotoLogin(null)
                UserLogin.clearAllAccount()
                return true
            }
            //车辆未解绑
            CARNOTUBOUNDCODE -> {
                normalDialog = NormalDialog(this@AccountCancellationActivity, NormalDialog.HORIZONTAL_TWO,
                        arrayOf(getString(R.string.cancel),getString(R.string.unregister_unbind)))
                normalDialog?.setClickButtonListener(object : HorizontalTwoButtonClick(normalDialog){
                    override fun onRightClick() {
                        var intent = Intent(this@AccountCancellationActivity,MyCarListActivity::class.java)
                        startActivity(intent)
                    }

                    override fun onLeftClick() {
                    }

                });
            }
            //有未完成订单
            UNFULFILLEDRDER -> {
                normalDialog = NormalDialog(this@AccountCancellationActivity, NormalDialog.VERTICAL_TWO,
                        arrayOf(getString(R.string.see_my_order), getString(R.string.cancel)))
                normalDialog?.setUseStyle(false, "#FF13D9C9", "#FF8F8E94", null)
                normalDialog?.setClickButtonListener(object : VerticalTwoButtonClick(normalDialog) {
                    override fun onBottomClick() {
                    }

                    override fun onTopClick() {
                        val basicWebData = BasicWebData("")
                        ARouter.getInstance()
                                .build(ARouterUrl.URL_MAIN_ORDER_LIST)
                                .withParcelable(BasicWebData.BASIC_WEB_DATA, basicWebData)
                                .navigation()
                    }
                })
            }
            //有车钱包余额
            CHECKBALANCE -> {
                normalDialog = NormalDialog(this@AccountCancellationActivity, NormalDialog.SINGLE,
                        arrayOf(getString(R.string.iknow)))
                normalDialog?.setClickButtonListener(object : SingleButtonClick(normalDialog) {
                    override fun onSingleClick() {

                    }
                })
            }
        }

        normalDialog?.setDialogTitle(getString(R.string.account_cancellation_fail))
        normalDialog?.tvDialogTitle?.visibility = View.VISIBLE
        normalDialog?.setDialogContent(checkCancellationAccountBean.message)
        normalDialog?.show()

        return isCheckSuccess
    }

    fun showAccountCancellactionFailDialog(dialgStyle: Int, names: Array<String>, content: String) {
        val normalDialog = NormalDialog(this@AccountCancellationActivity, dialgStyle,
                names)
        normalDialog.setDialogTitle(getString(R.string.account_cancellation_fail))
        normalDialog.tvDialogTitle.visibility = View.VISIBLE
        normalDialog.setDialogContent(content)
        if (dialgStyle == NormalDialog.VERTICAL_TWO) {
            normalDialog.setUseStyle(false, "#FF13D9C9", "#FF8F8E94", null)
            normalDialog.setClickButtonListener(object : VerticalTwoButtonClick(normalDialog) {
                override fun onBottomClick() {
                }

                override fun onTopClick() {
                    val basicWebData = BasicWebData("")
                    ARouter.getInstance()
                            .build(ARouterUrl.URL_MAIN_ORDER_LIST)
                            .withParcelable(BasicWebData.BASIC_WEB_DATA, basicWebData)
                            .navigation()
                }
            })
        } else if (dialgStyle == NormalDialog.SINGLE) {
            // normalDialog.setClickButtonListener(object : Clck)
        } else {
            normalDialog.setClickButtonListener(object : HorizontalTwoButtonClick(normalDialog) {

                override fun onLeftClick() {

                }

                override fun onRightClick() {
                    contactPhone(btCancellation, BrandHelper.getTelephoneForBm())
                }
            })
        }
        normalDialog.show()
    }

    fun contactPhone(view: View?, mobile: String) {
        val contactPhonePopupWindow = ContactPhonePopupWindow.show(this@AccountCancellationActivity, view)
        contactPhonePopupWindow.setTelephoneNumbers(mobile)
    }
}
