package org.zxq.teleri.activity

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.Uri
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import org.zxq.teleri.R
import org.zxq.teleri.bean.GetLastApkVersionData
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.utils.PackageUtil
import org.zxq.teleri.global.Constant
import org.zxq.teleri.global.MagicConstant
import org.zxq.teleri.secure.utils.SPUtils
import org.zxq.teleri.service.UpdateAPKService
import org.zxq.teleri.systemsetting.SystemSettingGiveGradePop
import org.zxq.teleri.ui.base.SimpleBaseActivity
import org.zxq.teleri.ui.utils.NetUtilUI
import org.zxq.teleri.ui.web.BaseWebActivity.TITLE_KEY
import org.zxq.teleri.utils.ZXQUtils
import java.util.ArrayList


/**
 * Created by leixiong on 2019/3/18.
 */
class SystemSettingAboutBmActivity : SimpleBaseActivity() {

    lateinit var rlGiveGrade: RelativeLayout
    lateinit var tvGiveGrade: TextView
    lateinit var ivGiveGrade: ImageView
    lateinit var mTvVersionName: TextView

    var mLastApkVersionData: GetLastApkVersionData?=null;
    var versionCode=0;

    lateinit var versionReceiver:BroadcastReceiver;

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initView()
        initData()

        requestVersion()
    }

    fun requestVersion(){
        var versionFilter =  IntentFilter();
        versionFilter.addAction(UpdateAPKService.ACTION_APK_NEW_VERSION);
        registerReceiver(versionReceiver, versionFilter);
        var service = Intent(this, UpdateAPKService::class.java);
        service.putExtra(MagicConstant.CHECK_VERSION, true);
        startService(service);
    }

    fun initView() {
        setContentView(R.layout.activity_about_bm)
        setTitle(org.zxq.teleri.R.string.about_bm)
        rlGiveGrade = findViewById(R.id.rl_give_grade)
        rlGiveGrade.setOnClickListener(this)
        tvGiveGrade = findViewById(R.id.tv_give_grade)
        ivGiveGrade = findViewById(R.id.iv_give_grade)
        mTvVersionName = findViewById(R.id.tv_version_name)

        findViewById<ImageView>(R.id.back).setOnClickListener(this)
        findViewById<RelativeLayout>(R.id.rl_version_introduction).setOnClickListener(this)
        findViewById<RelativeLayout>(R.id.rl_user_agree).setOnClickListener(this)
        findViewById<RelativeLayout>(R.id.rl_user_privacy).setOnClickListener(this)
        findViewById<RelativeLayout>(R.id.rl_contact_us).setOnClickListener(this)
        findViewById<RelativeLayout>(R.id.rl_system_setting_version_update).setOnClickListener(this)
    }

    fun initData() {
        versionReceiver = object:BroadcastReceiver(){
            override fun onReceive(context: Context?, intent: Intent?) {
                mLastApkVersionData = intent!!.getParcelableExtra("data");

                if (mLastApkVersionData != null) {
                    if (mLastApkVersionData!!.versionCode > versionCode) {
                        mTvVersionName.setCompoundDrawablesWithIntrinsicBounds(
                                ContextCompat.getDrawable(this@SystemSettingAboutBmActivity, R.drawable.set_new), null, null, null);
                    } else {
                        mTvVersionName.setCompoundDrawablesWithIntrinsicBounds(null, null, null, null);
                    }
                }
            }
        };
        getPackageInfo()
    }

    override fun onResume() {
        super.onResume()
        setGiveGradeType()
        getPackageInfo()
    }

    /**
     * 设置最新版本信息
     */
    fun getPackageInfo(){
        versionCode = PackageUtil.getVersionCode()
        val versionName = PackageUtil.getVersionName()
        mTvVersionName.text = versionName
        mTvVersionName.visibility = View.VISIBLE
    }

    override fun onClick(v: View?) {
        super.onClick(v)
        when (v?.id) {
            R.id.rl_version_introduction -> {//版本介绍
                startActivity(Intent(this, SystemSettingVersionIntroduction::class.java))
                saveDataRecord("edition")
            }
            R.id.rl_user_agree -> {//用户协议

                startUserAgreementWebActivity(Constant.BANMA, Constant.USER_AGREMENT_ADAPTER_TYPE,"用户协议")
            }
            R.id.rl_user_privacy -> {//用户隐私
                startUserAgreementWebActivity(Constant.BANMA, Constant.USER_PRIVACY_ADAPTER_TYPE,"隐私政策")
            }
            R.id.rl_give_grade -> {//去评分
                showGiveGrade()
                saveDataRecord("score")

            }
            R.id.rl_contact_us -> {//联系我们
                startActivity(Intent(this, SystemSettingContactUsActivity::class.java))
                saveDataRecord("contact")

            }
            R.id.rl_system_setting_version_update->{
                if (!NetUtilUI.isNetworkConnectedAndShowToast(true)) {
                    return;
                }

                if (null != mLastApkVersionData && mLastApkVersionData!!.versionCode > versionCode) {
                    var updateIntent = Intent(this, SystemSettingUpdateActivity::class.java);
                    updateIntent.putExtra(SystemSettingUpdateActivity.UPDATE_DATA, mLastApkVersionData);
                    startActivity(updateIntent);
                } else {
                    toast(R.string.new_version);
                }

                saveUpdate();
            }
        }
    }

    private fun startUserAgreementWebActivity(oemCode: String, adapterType: String,title:String) {
        var privacyInten = Intent(this, UserAgreementWebViewActivity::class.java)
        privacyInten.putExtra(Constant.OEMCODE, oemCode)
        privacyInten.putExtra(Constant.ADAPTERTYPE, adapterType)
        privacyInten.putExtra(TITLE_KEY,title)
        startActivity(privacyInten)


    }

    private fun saveDataRecord(controlName: String) {
        Framework.getDataRecord().ctrlClicked("app_set", controlName)
    }

    private fun showGiveGrade() {
        if (!NetUtilUI.isNetworkConnectedAndShowToast(true)) {
            return
        }
        val data = getInstalledStore()
        val packageName = SPUtils.getString(SPUtils.TO_GIVE_GRADE, SPUtils.TO_GIVE_GRADE_KEY, "")
        if (data.contains(packageName)) {
            launchAppDetail(packageName)
            return
        }
        SystemSettingGiveGradePop.show(this, rlGiveGrade, data)
    }

    private fun setGiveGradeType() {
        val packageNameData = getInstalledStore()
        if (packageNameData.isEmpty()) {
            tvGiveGrade.setTextColor(-0x2e2b26)
            ivGiveGrade.isPressed = true
            rlGiveGrade.isEnabled = false
        }
    }

    /**
     * 获取在手机上zxq发布的应用市场
     */
    private fun getInstalledStore(): List<String> {
        val installedStore = ArrayList<String>()
        for (i in SystemSettingGiveGradePop.BMSTOTELIST.indices) {
            if (ZXQUtils.isAppInstalled(SystemSettingGiveGradePop.BMSTOTELIST[i])) {
                installedStore.add(SystemSettingGiveGradePop.BMSTOTELIST[i])
            }
        }
        return installedStore
    }

    /**
     * 跳转到应用市场app详情界面
     * @param marketPkg 应用市场包名
     */
    fun launchAppDetail(marketPkg: String) {
        try {
            val uri = Uri.parse("market://details?id=$packageName")
            val intent = Intent(Intent.ACTION_VIEW, uri)
            if (!TextUtils.isEmpty(marketPkg)) {
                intent.setPackage(marketPkg)
            }
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            startActivity(intent)
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    fun saveUpdate() {
        Framework.getDataRecord().ctrlClicked("app_set", "update");
    }

    override fun onDestroy() {
        unregisterReceiver(versionReceiver)
        super.onDestroy()
    }

}