package org.zxq.teleri.activity

import android.content.Intent
import android.os.Bundle
import android.view.View
import com.ebanma.sdk.core.beans.DisclaimerBean
import com.ebanma.sdk.core.listener.BMResultCallback
import com.ebanma.sdk.core.net.BMApiImpl
import com.ebanma.sdk.core.net.exception.ApiException
import org.zxq.teleri.global.Constant
import org.zxq.teleri.login.UserPresenterImpl
import org.zxq.teleri.ui.R
import org.zxq.teleri.ui.utils.NetUtilUI
import org.zxq.teleri.ui.web.BaseWebActivity

/**
 * Created by leixiong on 2019/3/18.
 */
class UserAgreementWebViewActivity : BaseWebActivity() {
    companion object {
        const val NOT_LOGIN_ARG = "not_login_yet"
    }
    lateinit var oemCode: String
    lateinit var adapterType: String
    var notLogin: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        shareIcon.visibility = View.GONE
        findViewById<View>(R.id.system_bar).visibility = View.VISIBLE
        mInitData()
    }

    private fun mInitData() {
        oemCode = intent.getStringExtra(Constant.OEMCODE)
        adapterType = intent.getStringExtra(Constant.ADAPTERTYPE)
        mTitleString = intent.getStringExtra(TITLE_KEY)
        notLogin = intent.getBooleanExtra(NOT_LOGIN_ARG, false)
        if (NetUtilUI.isNetworkConnectedAndShowToast(false)) {
            requestAgreement(oemCode, adapterType)
        } else {
            showErrorView()
            mErrorView?.findViewById<View>(R.id.btn_reload)?.setOnClickListener { v ->
                if (NetUtilUI.isNetworkConnectedAndShowToast(false)) {
                    requestAgreement(oemCode, adapterType)
                }
            }
        }

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_SET_NETWORK) {
            if (NetUtilUI.isNetworkConnectedAndShowToast(false)) {
                requestAgreement(oemCode, adapterType)
            }
        }
    }

    private fun requestAgreement(oemCode: String, adapterType: String) {
        showLoadingDialog()
        getProtocolUrl(oemCode, adapterType)
    }

    override fun ignoreRestoreAspect(): Boolean {
        return true
    }

    private fun getProtocolUrl(oemCode: String, adapterType: String) {
        if (notLogin) {
            BMApiImpl.instance.disclaimerQueryWithoutLogin(oemCode, adapterType, object : BMResultCallback<String> {
                override fun onFail(apiException: ApiException) {
                    toast(apiException.message)
                    closeLoadingDialog()
                }

                override fun onSuccess(result: String) {
                    closeLoadingDialog()
                    mWebView.loadUrl(result)
                }
            })
        } else {
            BMApiImpl.instance.disclaimerQuery(oemCode, adapterType, object : BMResultCallback<DisclaimerBean> {
                override fun onFail(apiException: ApiException) {
                    toast(apiException.message)
                    closeLoadingDialog()
                    UserPresenterImpl.setDataAnalysis(oemCode, apiException.code.toString())
                }

                override fun onSuccess(result: DisclaimerBean) {
                    closeLoadingDialog()
                    mWebView.loadUrl(result.contentUrl)
                    UserPresenterImpl.setDataAnalysis(oemCode, "suc" + result.version)
                }
            })
        }
    }
}