/***************************************************************************************
 *
 * Project:        ZXQ
 *
 * Copyright ©     2014-2018 Banma Technologies Co.,Ltd
 * All rights reserved.
 *
 * This software is supplied only under the terms of a license agreement,
 * nondisclosure agreement or other written agreement with Banma Technologies
 * Co.,Ltd. Use, redistribution or other disclosure of any parts of this
 * software is prohibited except in accordance with the terms of such written
 * agreement with Banma Technologies Co.,Ltd. This software is confidential
 * and proprietary information of Banma Technologies Co.,Ltd.
 *
 *
 *
 * File Name: AlbumActivity.java
 *
 * General Description: Copyright and file header.
 *
 * Revision History:
 * Modification
 * Author                Date(MM/DD/YYYY)   JiraID           Description of Changes
 * ---------------------   ------------    ----------     -----------------------------
 * JohnFan              7/21/2018
 *
 */
package org.zxq.teleri.album

import android.graphics.Color
import android.os.Bundle
import android.text.Spannable
import android.text.SpannableString
import android.text.style.AbsoluteSizeSpan
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import kotlinx.android.synthetic.main.activity_album.*
import kotlinx.android.synthetic.main.layout_album_complete.*
import kotlinx.android.synthetic.main.layout_album_prepared.*
import kotlinx.android.synthetic.main.layout_album_sync_failed.*
import org.zxq.teleri.R
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.ui.adapter.ViewPagerAdapter
import org.zxq.teleri.ui.base.SimpleBaseActivity
import org.zxq.teleri.ui.utils.AppUtils
import org.zxq.teleri.ui.utils.UIUtils
import java.util.*

/**
 * 相册同步界面
 * @author JohnFan
 * @date 07/25/2018
 */
class AlbumActivity : SimpleBaseActivity(), AlbumContract.View {

    private lateinit var albumPresenter: AlbumContract.Presenter
    private var viewList: MutableList<View> = ArrayList()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_album)
        initTitleBar()
        initView()
        albumPresenter = AlbumPresenter.getInstance()
        albumPresenter.init(this)
        albumPresenter.start()
    }

    override fun onStart() {
        super.onStart()
        //同步状态
        when (albumPresenter.getCurrentStatus()) {
            AlbumContract.STATE_SYNCING -> connectState(AlbumContract.STATE_SYNCING, null)
            else -> albumPresenter.checkStatus()
        }
    }

    override fun onResume() {
        super.onResume()
        Framework.getDataRecord().enterPage("photosync_syncsucceed")
    }

    override fun onPause() {
        Framework.getDataRecord().leavePage("photosync_syncsucceed")
        super.onPause()
    }

    private fun initTitleBar() {
        setTitle(getString(R.string.album_title))
        my_title.findViewById<ImageView>(R.id.back).setOnClickListener {
            onBackPressed()
            when (btnStartSync.text) {
                getString(R.string.album_sync_again) -> ctrlClicked("photosync_syncsucceed", "photosync_syncsucceed_back")
                getString(R.string.album_sync_retry) -> ctrlClicked("photosync_syncfail", "photosync_syncfail_back")
                getString(R.string.album_sync_cancel) -> {
                    toast("取消成功")
                    ctrlClicked("photosync_syncing", "photosync_syncing_back")
                }
                getString(R.string.album_wifi_account_btn) -> ctrlClicked("photosync_switchguide", "photosync_switchguide_back")
                getString(R.string.album_wifi_closed_btn) -> ctrlClicked("photosync_wifiguide", "photosync_wifiguide_back")
            }

        }
    }

    private fun initView() {
        btnStartSync.setOnClickListener {
            when (btnStartSync.text) {
                getString(R.string.album_sync_again) -> ctrlClicked("photosync_syncsucceed", "photosync_syncsucceed_again")
                getString(R.string.album_sync_retry) -> ctrlClicked("photosync_syncfail", "photosync_syncfail_retry")
                getString(R.string.album_sync_cancel) -> {
                    toast("取消成功")
                    ctrlClicked("photosync_syncing", "photosync_syncing_cancel")
                }
                getString(R.string.album_sync_start) ->
                    ctrlClicked("photosync_main", "photosync_main_sync", "startsync")
                getString(R.string.album_wifi_account_btn) -> ctrlClicked("photosync_switchguide", "photosync_switchguide_set")
                getString(R.string.album_wifi_closed_btn) -> ctrlClicked("photosync_wifiguide", "photosync_wifiguide_set")

            }
            albumPresenter.clickedSync()
        }

        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_wificlosed, null))
        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_wifichange, null))
        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_prepared, null))
        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_sync, null))
        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_complete, null))
        viewList.add(LayoutInflater.from(this).inflate(R.layout.layout_album_sync_failed, null))
        albumViewPager.offscreenPageLimit = 6
        albumViewPager.adapter = ViewPagerAdapter(viewList)

    }

    override fun connectState(state: Int, ssId: String?) {
        LogUtils.i("connectState state=$state")
        closeLoadingDialog()
        when (state) {
            AlbumContract.STATE_PERMISSION_ALBUM -> {
                viewNoPermission!!.inflate()
                commitEvent("", "photosync_neterror")
            }
            AlbumContract.STATE_WIFI_DISCONNECTED -> {
                albumViewPager.setCurrentItem(0, false)
                updateSyncButton(getString(R.string.album_wifi_closed_btn))
                tvTips.visibility = View.INVISIBLE
                commitEvent("", "photosync_wifiguide")
            }
            AlbumContract.STATE_AUTHORITY_FAILED -> {
                albumViewPager.setCurrentItem(1, false)
                viewList[1].findViewById<TextView>(R.id.album_wifi_account).text = String.format(getString(R.string.album_wifi_account), ssId)
                updateSyncButton(getString(R.string.album_wifi_account_btn))
                tvTips.visibility = View.INVISIBLE
                commitEvent("", "photosync_switchguide")
            }
            AlbumContract.STATE_AUTHORITY_PREPARE -> {
                albumViewPager.setCurrentItem(2, false)
                updateSyncButton(getString(R.string.album_sync_start))
                tvTips.visibility = View.VISIBLE
                commitEvent("", "photosync_main")
            }
            AlbumContract.STATE_SYNC_PREPARE -> {
                albumViewPager.setCurrentItem(2, false)
                showLoadingDialog()
                updateSyncButton(getString(R.string.album_sync_prepare))
                tvTips.visibility = View.GONE
            }
            AlbumContract.STATE_SYNCING -> {
                albumViewPager.setCurrentItem(3, false)
                updateSyncButton(getString(R.string.album_sync_cancel))
                tvTips.visibility = View.VISIBLE
                syncSuccessed()
                commitEvent("", "photosync_syncing")
            }
            AlbumContract.STATE_SYNC_COMPLETE -> {
                albumViewPager.setCurrentItem(4, false)
                updateSyncButton(getString(R.string.album_sync_again))
                tvTips.visibility = View.VISIBLE
                commitEvent("", "photosync_syncsucceed")
            }
            AlbumContract.STATE_SYNC_FAILED -> {
                albumViewPager.setCurrentItem(5, false)
                updateSyncButton(getString(R.string.album_sync_retry))
                tvTips.visibility = View.VISIBLE
                commitEvent("", "photosync_syncfail")
            }
        }
    }

    override fun showErrorToast(id: Int) {
        AppUtils.showToast(id)
    }

    override fun syncPrepared(total: Int) {
        closeLoadingDialog()
        when (total) {
            0 -> {
                btnStartSync.isEnabled = false
                tvTotal.text = getString(R.string.album_nothing_to_sync)
            }
            // 代表未加载出来
            -1 -> tvTotal.text = ""
            else -> tvTotal.text = String.format(getString(R.string.album_prepare), total)
        }
    }


    override fun syncStateChanged(currentPic: Int, successes: Boolean) {

    }

    override fun syncSuccessed() {
        viewList[3].findViewById<TextView>(R.id.tvSyncProcess).text = albumPresenter.getSyncProcess()
//        tvSyncProcess?.text = albumPresenter.getSyncProcess()
    }

    override fun syncFailed(notSyncFile: Int) {
        val string = SpannableString(String.format(getString(R.string.album_sync_error_files), notSyncFile))
        val startIndex = 0
        string.setSpan(AbsoluteSizeSpan(UIUtils.dip2px(17.28f)), startIndex, startIndex + notSyncFile.toString().length, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        string.setSpan(ForegroundColorSpan(Color.parseColor("#FFC52020")), startIndex, startIndex + notSyncFile.toString().length, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        tvNotSyncedFiles.text = string
    }

    private fun updateSyncButton(text: String) {
        btnStartSync.text = text
    }

    override fun setPresenter(presenter: AlbumContract.Presenter) {
        albumPresenter = presenter
    }

    override fun syncCompleted(syncedFile: Int) {
        val startIndex = 3
        val string = SpannableString(String.format(getString(R.string.album_synced), syncedFile))
        string.setSpan(AbsoluteSizeSpan(UIUtils.dip2px(17.28f)), startIndex, startIndex + syncedFile.toString().length, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        string.setSpan(ForegroundColorSpan(Color.parseColor("#13D9C9")), startIndex, startIndex + syncedFile.toString().length, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        tvSyncedFiles.text = string
    }

    override fun onBackPressed() {
        super.onBackPressed()
        this@AlbumActivity.finish()
    }
}
