/***************************************************************************************
 *
 *  Project:        ZXQ
 *
 *  Copyright ©     2014-2018 Banma Technologies Co.,Ltd
 *                  All rights reserved.
 *
 *  This software is supplied only under the terms of a license agreement,
 *  nondisclosure agreement or other written agreement with Banma Technologies
 *  Co.,Ltd. Use, redistribution or other disclosure of any parts of this
 *  software is prohibited except in accordance with the terms of such written
 *  agreement with Banma Technologies Co.,Ltd. This software is confidential
 *  and proprietary information of Banma Technologies Co.,Ltd.
 *
 ***************************************************************************************
 *
 *  File Name: AlbumContract.java
 *
 *  General Description: Copyright and file header.
 *
 *  Revision History:
 *                           Modification
 *   Author                Date(MM/DD/YYYY)   JiraID           Description of Changes
 *   ---------------------   ------------    ----------     -----------------------------
 *   John Fan              7/21/2018
 *
 ****************************************************************************************/
package org.zxq.teleri.album

import org.zxq.teleri.base.BasePresenter
import org.zxq.teleri.base.BaseView

/**
 * 相册约定
 * @author JohnFan
 */
interface AlbumContract {

    interface View : BaseView<Presenter> {

        /**
         * 连接状态变化
         *
         * @param state
         * @param ssId  连接wifi的ssId
         */
        fun connectState(state: Int, ssId: String?)
        /**
         * 连接成功
         *
         * @param total 待同步文件数量
         */
        fun syncPrepared(total: Int)

        /**
         * 同步中状态
         *
         * @param currentPic 当前同步的文件序号
         * @param successes  当前同步文件成功失败
         */
        fun syncStateChanged(currentPic: Int, successes: Boolean)

        /**
         * 同步成功
         *
         */
        fun syncSuccessed()

        fun syncCompleted(syncedFile: Int)

        /**
         * 同步失败
         *
         * @param total 失败文件数量
         */
        fun syncFailed(notSyncFile: Int)

        fun showErrorToast(id:Int)
    }

    interface Presenter : BasePresenter {

        fun init(_albumView: View)
        /**
         * 检查当前网络状态
         *
         * @return wifi是否连接到车机
         */
        fun checkStatus()

        /**
         * @return WIFI SSID
         */
        fun getWifiSSID(): String

        /**
         * 按钮被点击
         */
        fun clickedSync()

        /**
         * 准备同步，获取待同步文件数量
         * */
        fun prepareSync()

        /**
         * 开始同步车机数据
         */
        fun startSync()

        /**
         * 停止车机数据
         */
        fun stopSync()

        /**
         * 当前同步状态
         *
         */
        fun getCurrentStatus(): Int

        /**
         * 得到同步中的数量
         */
        fun getSyncProcess(): String


    }

    companion object {

        /**
         * 缺少写入相册权限
         */
        const val STATE_PERMISSION_ALBUM = 0
        /**
         * wifi已断开
         */
        const val STATE_WIFI_DISCONNECTED = 1
        /**
         * 车机端账号不匹配
         */
        const val STATE_AUTHORITY_FAILED = 2
        /**
         * 车机端账号验证成功 ，返回待同步数量
         */
        const val STATE_AUTHORITY_PREPARE = 3
        /**
         * 同步中
         */
        const val STATE_SYNCING = 4
        /**
         * 同步完成
         */
        const val STATE_SYNC_COMPLETE = 5
        /**同步失败*/
        const val STATE_SYNC_FAILED = 6
        //手机端正向车机发起连接，车机端还没有返回当前待同步的数量
        const val STATE_SYNC_PREPARE=7
    }

}
