/***************************************************************************************
 *
 * Project:        ZXQ
 *
 * Copyright ©     2014-2018 Banma Technologies Co.,Ltd
 * All rights reserved.
 *
 * This software is supplied only under the terms of a license agreement,
 * nondisclosure agreement or other written agreement with Banma Technologies
 * Co.,Ltd. Use, redistribution or other disclosure of any parts of this
 * software is prohibited except in accordance with the terms of such written
 * agreement with Banma Technologies Co.,Ltd. This software is confidential
 * and proprietary information of Banma Technologies Co.,Ltd.
 *
 *
 *
 * File Name: AlbumPresenter.java
 *
 * General Description: Copyright and file header.
 *
 * Revision History:
 * Modification
 * Author                Date(MM/DD/YYYY)   JiraID           Description of Changes
 * ---------------------   ------------    ----------     -----------------------------
 * John Fan              7/21/2018
 *
 */
package org.zxq.teleri.album

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.net.wifi.WifiManager
import android.os.Environment
import android.preference.PreferenceManager
import android.util.Log
import com.ebanma.sdk.core.utils.OkIOUtil
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import org.zxq.teleri.R
import org.zxq.teleri.core.utils.*
import org.zxq.teleri.ui.utils.AppUtils
import java.io.File
import java.util.concurrent.TimeUnit

/**
 * @author JohnFan
 * @date 07/25/2018
 */
class AlbumPresenter private constructor() : AlbumContract.Presenter {

    private lateinit var albumView: AlbumContract.View
    private var totalToSync: Int = 0
    private var mAlbumListBean: AlbumListBean? = null
    private lateinit var okHttpClient: OkHttpClient
    private var isStopped = false
    private var autoStart = false
    private var isSyncing = false
    private var syncProcess = ""

    lateinit var albumSyncManager: AlbumSyncManager

    var currentState = AlbumContract.STATE_WIFI_DISCONNECTED

    companion object {
        const val TAG = "AlbumPresenter"
        const val ALBUM_LAST_SYNCED_ID = "album_last_synced_id"
        @JvmStatic
        fun getInstance() = Holder.INSTANCE

    }

    override fun getCurrentStatus(): Int {
        return currentState
    }

    private object Holder {
        val INSTANCE = AlbumPresenter()
    }


    override fun init(_albumView: AlbumContract.View) {
        this.albumView = _albumView
        autoStart = false
        albumView.setPresenter(this)
        // 启动信息流转服务，准备与车机交互数据
        //初始化建立信息流转
        albumSyncManager = AlbumSyncManager.getInstance()
        albumSyncManager.init(ContextPool.getApplication()) {
            //            checkStatus()
        }

        try {
            albumSyncManager.connectToVehicle()
        } catch (e: Exception) {
            e.printStackTrace()
        }

        currentState = if (!isWiFiConnectedCar()) {
            LogUtils.d(TAG, "没连上车机wifi")
            albumView.connectState(AlbumContract.STATE_WIFI_DISCONNECTED, null)
            AlbumContract.STATE_WIFI_DISCONNECTED
        } else {
            // 先设置状态为loading
            albumView.connectState(AlbumContract.STATE_SYNC_PREPARE, null)
            AlbumContract.STATE_SYNC_PREPARE
        }

        initRetrofit()
    }

    @SuppressLint("WifiManagerLeak")
    private fun isWiFiConnectedCar(): Boolean {
        val wifiManager = ContextPool.getApplication()?.getSystemService(Context.WIFI_SERVICE) as WifiManager
        val wifiInfo = wifiManager.connectionInfo
        val isContainIOV = wifiInfo.ssid.contains("IOV")
        return NetUtil.checkWifi(ContextPool.getApplication()) && isContainIOV
    }

    private fun initRetrofit() {
        try {
            okHttpClient = OkHttpClient().newBuilder()
                    .connectTimeout(10, TimeUnit.SECONDS).readTimeout(10, TimeUnit.SECONDS).build()
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


    override fun start() {

    }

    /**
     * 初始化连接状态
     */
    override fun checkStatus() {

        val lastSyncId = getInt(ALBUM_LAST_SYNCED_ID)

        try {
            AlbumSyncManager.getInstance().sendAlbumSyncRequestWithLastSyncId(lastSyncId, object : AlbumSyncManager.CompletionHandler {

                override fun onNotConnectCarWiFi() {
                    LogUtils.d(TAG, "没连上车机wifi")
                    albumView.connectState(AlbumContract.STATE_WIFI_DISCONNECTED, null)
                    currentState = AlbumContract.STATE_WIFI_DISCONNECTED
                }

                override fun onDifferentAccount() {
                    LogUtils.d(TAG, "账户不一致")
                    albumView.connectState(AlbumContract.STATE_AUTHORITY_FAILED, getWifiSSID())
                    currentState = AlbumContract.STATE_AUTHORITY_FAILED
                }

                override fun onConnectWithSameAccount() {
                    LogUtils.d(TAG, "正确连上车机wifi,开始发起相册文件请求")
                }

                override fun onGetDataLoading() {
                    currentState = AlbumContract.STATE_SYNC_PREPARE
                    albumView.connectState(AlbumContract.STATE_SYNC_PREPARE, getWifiSSID())
                }

                override fun onGetDataSuccess(albumListBean: AlbumListBean?) {
                    mAlbumListBean = albumListBean
                    LogUtils.i(TAG, albumListBean?.count.toString())
                    currentState = AlbumContract.STATE_AUTHORITY_PREPARE

                    albumView.connectState(AlbumContract.STATE_AUTHORITY_PREPARE, getWifiSSID())

                    totalToSync = albumListBean?.data?.size ?: 0
                    LogUtils.d(TAG, "totalToSync = $totalToSync")
                    albumView.syncPrepared(totalToSync)
                    if (autoStart && totalToSync > 0) {
                        startSync()
                    }
                }

                override fun onGetDataFailure() {
                    LogUtils.d(TAG, "获取相册数量接口失败")
                }
            })
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


    override fun clickedSync() {
        when (currentState) {
            AlbumContract.STATE_WIFI_DISCONNECTED, AlbumContract.STATE_AUTHORITY_FAILED -> SettingsUtil.startWirelessSetting(ContextPool.getApplication())

            AlbumContract.STATE_SYNC_PREPARE -> AppUtils.showToast("查询中，请稍后")
            //开始同步
            AlbumContract.STATE_AUTHORITY_PREPARE -> startSync()
            //手动取消同步
            AlbumContract.STATE_SYNCING -> stopSync()

            AlbumContract.STATE_SYNC_COMPLETE -> {
                albumView.connectState(AlbumContract.STATE_SYNC_PREPARE, null)//再次同步
                checkStatus()
            }

            AlbumContract.STATE_SYNC_FAILED -> {
                autoStart = true
                checkStatus()
            }
        }
    }


    override fun getWifiSSID(): String = NetUtil.getWifiSSID(ContextPool.getApplication())

    override fun prepareSync() {
        albumView.connectState(AlbumContract.STATE_AUTHORITY_PREPARE, null)
        currentState = AlbumContract.STATE_AUTHORITY_PREPARE
    }


    fun setPreference(key: String, value: Int?) {
        var value = value
        if (value == null) {
            value = 0
        }
        PreferenceManager.getDefaultSharedPreferences(ContextPool.getApplication()).edit().putInt(key, value).apply()
    }

    /**
     * 默认值为0
     */
    fun getInt(key: String): Int {
//        return 0//test
        return PreferenceManager.getDefaultSharedPreferences(ContextPool.getApplication()).getInt(key, 0)
    }


    fun getSaveDir(): String {
        val dir = Environment.getExternalStorageDirectory().absolutePath + File.separator + "斑马相册同步"
        val file = File(dir)
        if (!file.exists()) {
            file.mkdir()
        }
        return file.absolutePath
    }

    override fun startSync() {

        isStopped = false
        var totalSize = 0.0
        mAlbumListBean?.data?.forEach { bean ->
            totalSize += bean.fileSize.toDoubleOrNull() ?: 0.0
        }
        // 判断手机空间，如果不够，toast提示
        val freeSd = SdcardUtil.getFreeSpaceOnSdcardKB()
        if (totalSize > freeSd) {
            albumView.showErrorToast(R.string.album_no_space)
            return
        }

        //界面切换到正在同步中
        albumView.connectState(AlbumContract.STATE_SYNCING, null)
        currentState = AlbumContract.STATE_SYNCING

        syncProcess = "0/$totalToSync"
        albumView.syncSuccessed()
        var index = 1
        var isError = false
        ThreadUtils.runOnIoThreadWithMainCallBack(object : ThreadUtils.ThreadAction {
            override fun ioAction() {
                isSyncing = true
                // 循环下载所有文件
                mAlbumListBean?.data?.forEach { bean ->
                    if (isStopped) {
                        LogUtils.w(TAG, "Stopped download.")
                        return
                    }
                    LogUtils.i(TAG, "start download ${bean.id} ${getFileName(bean.filePath)}")
                    val url: String = bean.filePath
                    val request = Request.Builder().url(url).build()
                    val call = okHttpClient.newCall(request)
                    val fileType = bean.fileName.subSequence(bean.fileName.lastIndexOf(".") + 1, bean.fileName.length)
                    LogUtils.w(TAG, "fileType:$fileType")
                    val filePath = getSaveDir() + File.separator + "${System.currentTimeMillis()}." + fileType

                    val response: Response
                    try {
                        response = call.execute()
                        // for test
//                        if (index == 3) throw Exception()
                    } catch (e: Exception) {
                        e.printStackTrace()
                        // 下载超时提示"无响应，请重试"
                        albumView.showErrorToast(R.string.album_sync_error)
                        isError = true
                        return
                    }
                    try {
                        writeFile2Disc(response, filePath)
                        //记录成功的id
                        setPreference(ALBUM_LAST_SYNCED_ID, bean.id)
                        Log.d(TAG, "saved success id = ${bean.id}")
                    } catch (e: Exception) {
                        e.printStackTrace()
                        Log.d(TAG, "保存文件出错 $filePath")
                        isError = true
                        //出错的话，就把文件删了
                        File(filePath).delete()
                        return
                    }
                    ThreadUtils.runOnMainThread {
                        syncProcess = "$index/$totalToSync"
                        albumView.syncSuccessed()
                        index++
                    }
                }
            }

            override fun mainAction() {
                when {
                    isStopped -> {
                        prepareSync()
                    }
                    isError -> {
                        //切换到错误状态
                        albumView.connectState(AlbumContract.STATE_SYNC_FAILED, null)
                        albumView.syncFailed(totalToSync - (--index))
                        currentState = AlbumContract.STATE_SYNC_FAILED
                    }
                    else -> {
                        albumView.syncCompleted(--index)
                        //切换到同步完成
                        albumView.connectState(AlbumContract.STATE_SYNC_COMPLETE, null)
                        currentState = AlbumContract.STATE_SYNC_COMPLETE
                    }
                }
                isSyncing = false
            }

        })

    }

    /**
     * 保存文件并通知相册
     * */
    private fun writeFile2Disc(response: Response, path: String) {
        val ins = response.body?.byteStream()
        ins?.let {
            val file = File(path)
            OkIOUtil.writeStream2File(file.absolutePath, ins)
            LogUtils.i(TAG, "下载成功 " + file.absolutePath)

            //通知相册更新
            val intent = Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE)
            val uri = Uri.fromFile(file)
            intent.data = uri
            ContextPool.getApplication()?.sendBroadcast(intent)
        }
    }

    override fun stopSync() {
        currentState = AlbumContract.STATE_AUTHORITY_PREPARE
        albumView.connectState(AlbumContract.STATE_AUTHORITY_PREPARE, null)
//        albumView.syncPrepared(-1)
        autoStart = false
        isStopped = true
        checkStatus()
    }

    override fun destroy() {
        AlbumSyncManager.getInstance().release()
    }

    private fun getFileName(type: String) = type.split("/").last()

    override fun getSyncProcess(): String = syncProcess

}
