/*
 *
 *  *
 *  *  * **************************************************************************************
 *  *  *  *
 *  *  *  *  Project:        ZXQ
 *  *  *  *
 *  *  *  *  Copyright ©     2014-2018 Banma Technologies Co.,Ltd
 *  *  *  *                  All rights reserved.
 *  *  *  *
 *  *  *  *  This software is supplied only under the terms of a license agreement,
 *  *  *  *  nondisclosure agreement or other written agreement with Banma Technologies
 *  *  *  *  Co.,Ltd. Use, redistribution or other disclosure of any parts of this
 *  *  *  *  software is prohibited except in accordance with the terms of such written
 *  *  *  *  agreement with Banma Technologies Co.,Ltd. This software is confidential
 *  *  *  *  and proprietary information of Banma Technologies Co.,Ltd.
 *  *  *  *
 *  *  *  ***************************************************************************************
 *  *  *  *
 *  *  *  *  Header Name:
 *  *  *  *
 *  *  *  *  General Description: Copyright and file header.
 *  *  *  *
 *  *  *  *  Revision History:
 *  *  *  *                           Modification
 *  *  *  *   Author  Deman.LU        Date(MM/DD/YYYY)   JiraID       Description of Changes
 *  *  *  *
 *  *  *  *
 *  *  *  ***************************************************************************************
 *  *
 *
 */

package org.zxq.teleri.pay.alibaba

import android.app.Activity
import android.content.Context
import android.text.TextUtils
import com.alipay.sdk.app.PayTask
import org.zxq.teleri.alipay.CheckPayResult
import org.zxq.teleri.compoentservice.pay.IPayProvider
import org.zxq.teleri.compoentservice.pay.OnPayResultListener
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.core.utils.ThreadUtils


/**
 *   @author     Deman.LU
 *   @date       2019/4/22
 *
 */
class AliPayPorvider:IPayProvider {

    lateinit var activity:Activity
    lateinit var payUrl:String
    lateinit var payOrderId: String
    var payResultListener:OnPayResultListener? = null
    var rollNumber= 0

    override fun pay(activity: Activity,payOrderId:String, pay_redirect_url: String, onPayResultListener: OnPayResultListener) {
        LogUtils.d(pay_redirect_url)
        this.activity = activity
        this.payOrderId = payOrderId
        this.payUrl = pay_redirect_url
        this.payResultListener = onPayResultListener
        startPay()
    }

    private fun startPay() {
        LogUtils.i("startPay")
        var result:Map<String,String>? = mutableMapOf()
        ThreadUtils.runOnIoThreadWithMainCallBack(object:ThreadUtils.ThreadAction{
            override fun ioAction() {
                val alipay = PayTask(activity)
                // use pay interface, to obtain payment results
                result = alipay.payV2(payUrl, true)
            }

            override fun mainAction() {
                if(result.isNullOrEmpty()){
                    payResultListener?.onPayUnknow()
                }else{
//                    LogUtils.map(result!!)
                    parseResult(result!!)
                }
            }
        })
    }

    private fun parseResult(result: Map<String, String>) {
        val payResult = PayResult(result)
        val resultInfo = payResult.result
        val resultStatus = payResult.resultStatus

        LogUtils.i(resultStatus?:"")
        LogUtils.d(resultInfo?:"")

        /*resultStatus is “9000” for success*/
        when {
            TextUtils.equals(resultStatus, "9000") -> payResultListener?.onPaySucceed()
            /*
            // ResultStatus is not a "9000" represents may pay fail
            // "8000" is for pay the result or because the payment
            // channel system pay reason
            // is still waiting for the results confirmed that the final
            // transaction success
            // will be subject to the server-side asynchronous
            // notification (state) small probability
            */
            TextUtils.equals(resultStatus, "8000") -> payResultListener?.onPayUnconfirmed()
            // ResultStatus is "4000" that represents pay fail
            TextUtils.equals(resultStatus, "4000") -> CheckPayResult(payOrderId,{payResultListener?.onPaySucceed()},{payResultListener?.onPayFailed()}).queryPayResult()
            TextUtils.equals(resultStatus, "6001") -> payResultListener?.onPayCancled()
            else -> payResultListener?.onPayUnknow()
        }
    }

    override fun init(context: Context?) {

    }

//    /**
//     * 轮询结果
//     */
//    fun queryPayResult(){
//        rollNumber++
//        if(rollNumber>=4){
//            payResultListener?.onPayFailed()
//        }else{
//            GetPayResultRequest(payOrderId,1).subscribe({
//                s->
//                    var result: PayResultBean? = Json.from(s, PayResultBean::class.java)
//                    //PayStatus 100未支付，101支付成功，102支付失败
//                    if (result!!.payStatus == 101) {//成功
//                        LogUtils.i("queryPayResult-->success >>>" + rollNumber)
//                        payResultListener?.onPaySucceed()
//                    } else {//失败
//                        LogUtils.i("queryPayResult-->failed >>>" + rollNumber)
//                        Observable.just(1).delay(10,TimeUnit.SECONDS).subscribe {
//                            LogUtils.i("req time",""+System.currentTimeMillis())
//                            queryPayResult()
//                        }
//                    }
//            },{t ->
//                    LogUtils.i("queryPayResult>>>>>"+rollNumber+"，exception")
//                    t.printStackTrace()
//                    LogUtils.i("resp time",""+System.currentTimeMillis())
//                    //延迟5秒
//                    Observable.just(1).delay(10,TimeUnit.SECONDS).subscribe {
//                        LogUtils.i("req time",""+System.currentTimeMillis())
//                        queryPayResult()
//                    }
//                })
//        }
//    }

}