package org.zxq.teleri.bindcar

import android.arch.lifecycle.Observer
import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.View
import android.widget.Button
import android.widget.TextView
import org.zxq.teleri.R
import org.zxq.teleri.account.MyCarDetailActivity
import org.zxq.teleri.account.MyCarListActivity
import org.zxq.teleri.core.Framework
import org.zxq.teleri.mainentrance.MainPresenter
import org.zxq.teleri.ui.base.SimpleBaseActivity
import org.zxq.teleri.ui.utils.CountDownUtil
import org.zxq.teleri.ui.utils.ToastUtil
import org.zxq.teleri.ui.utils.UIUtils
import org.zxq.teleri.ui.widget.ClearEditText

class InputCode4UnbindCarActivity:SimpleBaseActivity() {

    private lateinit var mCountDownUtil: CountDownUtil
    lateinit var presenter:UnBindCarPresenter
    private var owner:Boolean =false
    lateinit var vin:String
    lateinit var mobile:String

    private lateinit var mGetCode: TextView
    private lateinit var mCode:ClearEditText
    lateinit var mUnbindBtn:Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_input_code_4_unbind_car)

        presenter = UnBindCarPresenter()

        initView()
        initData()
        bindEvent()
    }

    fun initView(){
        mGetCode = findViewById<TextView>(R.id.get_code)
        mCode = findViewById(R.id.code)
        mUnbindBtn = findViewById(R.id.unbind_btn)

    }

    fun initData(){
        owner = intent.getBooleanExtra("owner",false)
        vin = intent.getStringExtra("vin")
        mobile = intent.getStringExtra("mobile")

        if(owner){
            setTitle(R.string.unbind_title_4_self)
        }else{
            setTitle(R.string.unbind_title_4_other)
        }

        mCountDownUtil = CountDownUtil(mGetCode){
            btLoginClickable()
        };
        mCountDownUtil.start();
    }

    private fun bindEvent(){
        findViewById<View>(R.id.back).setOnClickListener(this)
        mGetCode.setOnClickListener{v->getCode()}
        mCode.addTextChangedListener(object : TextWatcher{
            override fun afterTextChanged(s: Editable?) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                mUnbindBtn.isEnabled = !TextUtils.isEmpty(s)
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }
        })
        mUnbindBtn.setOnClickListener{v->unBindCar()}
    }

    fun getCode(){
        showLoadingDialog()
        presenter.getCode4UnbindCar(owner,vin,mobile).observe(this, Observer {
            if(it!!){
                closeLoadingDialog()
                if(!it){
                    mCountDownUtil.onFinish()
                }else{
                    mCountDownUtil.start()
                    UIUtils.shortToast(R.string.verifycode_send_success)
                }
            }
        })
    }

    private fun unBindCar(){
        showLoadingDialog()
        presenter.unBindCar(owner,vin,mobile,mCode.text.toString(),"").observe(this, Observer {
            if(it!=null){
                closeLoadingDialog()
                if(it){
                    if(owner) {
                        var intent = Intent(this, MyCarListActivity::class.java)
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                        intent.putExtra("vin", vin)

                        //解绑的是否是当前车辆
                        if (vin.equals(Framework.getAccountInject().getVin())) {
                            intent.putExtra("UNBIND_CAR_FLAG", "unBindCurrentCar")
                        } else {
                            intent.putExtra("UNBIND_CAR_FLAG", "unBindCar")
                        }
                        startActivity(intent)

                        ToastUtil.show("解绑成功")
                    }else{
                        MainPresenter.getInstance().notifyObserverID(MyCarListActivity.OBSERVER_KEY_ID,
                                MyCarListActivity.KEY_GET_UNBIND_CAR_INFO_DATA, null)
                        val intent = Intent(this, MyCarDetailActivity::class.java)
                        intent.putExtras(getIntent().extras!!)
                        intent.putExtra("is_apply_unbind", true)
                        intent.putExtra("by_phone", true)
                        startActivity(intent)
                    }
                }
            }
        })
    }

    private fun btLoginClickable() {
        mUnbindBtn.isEnabled = !TextUtils.isEmpty(mCode.text.toString())
        mGetCode.isEnabled = !mCountDownUtil.runStatus
    }
}