package org.zxq.teleri.homepage.cardetail.viewinter

import io.reactivex.Observable
import org.zxq.teleri.account.datamodel.UserLogin
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.net.request.RequestA
import org.zxq.teleri.model.request.OemAPI
import org.zxq.teleri.model.request.OemRequest
import org.zxq.teleri.model.request.OpenAPI
import org.zxq.teleri.model.request.open.VinListRequest
import com.ebanma.sdk.core.domain.DomainUtil

/**
 *Created by cyc on 2019/3/11.
 *
 * 首页车控涉及的所有接口汇总
 */


class CarRepository {

    companion object {

        @Volatile
        private var instance: CarRepository? = null

        fun getInstance() = instance ?: synchronized(this) {
            instance ?: CarRepository().also { instance = it }
        }

        //屏蔽数据过滤
        private const val DATA_FIELD_FLAG = "vvtest"

        private const val DATA_FIELD_DATA = "data"
    }


    /**
     * -------------------------------------------------------------------------------------------------
     * 获取绑定的车辆列表
     */
    fun getVinList(): Observable<String> = VinListRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 车型权限适配表下载
     */
    internal class GetFunctionCodeRequest : RequestA() {
        override fun getApiName() = OpenAPI.BM_GET_FUNCTION_CODE

        override fun getParams() = mapOf(
                "accountId" to (UserLogin.getAccountA().id).toString(),
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = super.findDataField()
    }

    fun getFunctionCode(): Observable<String> = GetFunctionCodeRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 请求车辆信息（绑定 激活相关）
     */
    internal class GetCarInfoDataRequest : RequestA() {
        override fun getApiName() = OpenAPI.FIND_VEHICLEINFO_A

        override fun getParams() = mapOf(
                "token" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin,
                "userId" to (UserLogin.getAccountA().id).toString()
        )

        override fun getDataField(): String = super.findDataField()
    }

    fun getCarInfo() = GetCarInfoDataRequest().prepare()

    /**
     * -------------------------------------------------------------------------------------------------
     * 请求用户蓝牙钥匙列表 TODO 首页的授权蓝牙界面需要判断
     */
    internal class GetBlueKeyListRequest : OemRequest() {
        override fun getBizType() = DomainUtil.BizType.user

        override fun getApiName() = OemAPI.LIST_USER_BT_KEY_WITH_VIN

        override fun getParams() = mapOf(
                "token" to DomainUtil.getToken(DomainUtil.BizType.user),
                "oemUserId" to UserLogin.getAccountB().user_id,
                "timestamp" to System.currentTimeMillis(),
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = DATA_FIELD_FLAG
    }

    fun getBlueKeyList() = GetBlueKeyListRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 获取车辆被盗报状态
     */
    internal class GetCarStolenRequest : RequestA() {
        override fun getApiName() = OpenAPI.KEY_GET_VEHICLE_STOLEN_STATUS

        override fun getParams() = mapOf(
                "token" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = super.findDataField()
    }

    fun getCarStolenStatus() = GetCarStolenRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * request vehicle status 请求车辆状态上报  后续会收到TBOX的推送，根据推送要再获取当前车辆状态后刷新显示
     */
    internal class VehicleStatusRequest : RequestA() {
        override fun getApiName() = OpenAPI.KEY_REQUEST_VEHICLE_STATUS

        override fun getParams() = mapOf(
                "token" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = super.findDataField()
    }

    fun requestVehicleStatus(): Observable<String> = VehicleStatusRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 获取车辆状态接口
     */
    internal class GetVehicleStatusRequest : RequestA() {
        override fun getApiName() = OpenAPI.KEY_GET_VEHICLE_STATUS

        override fun getParams() = mapOf(
                "token" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = super.findDataField()
    }

    fun getVehicleStateData(): Observable<String> = GetVehicleStatusRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 活动的未读消息需要接口支持 其余读取本地数据库监听 未读消息初始监听
     */
    internal class GetCmsListRequest : RequestA() {
        override fun getApiName() = OpenAPI.GET_CMS_ADV_4B

        override fun getParams() = mapOf(
                "token" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin,
                "portal_id" to "M00P07",
                "limit" to "-1"//返回广告记录数，-1返回全部

        )

        override fun getDataField() = DATA_FIELD_DATA
    }

    fun getCmsListRequest(): Observable<String> = GetCmsListRequest().prepare()

    /**
     * -------------------------------------------------------------------------------------------------
     * 车辆保养接口相关信息
     */
    internal class GetCarMaintainRequest : OemRequest() {
        override fun getBizType() = DomainUtil.BizType.as_bm

        override fun getApiName() = OemAPI.FIND_PARTS_SCORE

        override fun getParams() = mapOf(
                "token" to DomainUtil.getToken(DomainUtil.BizType.as_bm),
                "timestamp" to System.currentTimeMillis(),
                "vin" to Framework.getAccountInject().vin,
                "icon_type" to "M01"
        )

        override fun getApiVersion() = "1.6"

        override fun getDataField(): String = DATA_FIELD_FLAG
    }

    fun getCarMaintain() = GetCarMaintainRequest().prepare()


    /**
     * -------------------------------------------------------------------------------------------------
     * 获取车控配置 ，运行时间
     */
    internal class GetVpProfileRequest : OemRequest() {
        override fun getBizType() = DomainUtil.BizType.vp

        override fun getApiName() = OemAPI.URL_GET_CARCONTROL_CFG

        override fun getParams() = mapOf(
                "token" to DomainUtil.getToken(DomainUtil.BizType.vp),
                "timestamp" to System.currentTimeMillis(),
                "vin" to Framework.getAccountInject().vin
        )

        override fun getDataField(): String = DATA_FIELD_DATA
    }

    fun getVpProfile(): Observable<String> = GetVpProfileRequest().prepare()


    /**
     * 获取云端保存温度
     */
    internal class TemSaveMessageRequest : RequestA() {
        override fun getApiName() = OpenAPI.KEY_TEMPERATURE_GET

        override fun getParams() = mapOf(
                "amToken" to UserLogin.getAccountA().token,
                "vin" to Framework.getAccountInject().vin

        )

        override fun getDataField(): String = super.findDataField()

        override fun getApiVersion() = "2.0"
    }

    fun getTemSaveMessageRequest(): Observable<String> = TemSaveMessageRequest().prepare()


}