package org.zxq.teleri.homepage.cardetail.viewinter

import android.app.Activity
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.ViewModel
import android.arch.lifecycle.ViewModelProvider
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import com.irdeto.securesdk.ss.SSUtils
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.functions.Function3
import io.reactivex.schedulers.Schedulers
import org.greenrobot.eventbus.EventBus
import org.zxq.teleri.R
import org.zxq.teleri.ZXQApplication
import org.zxq.teleri.account.MyCarListActivity
import org.zxq.teleri.account.datamodel.UserLogin
import org.zxq.teleri.bean.*
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.constant.FunctionCode
import org.zxq.teleri.core.ex.OnErrorResponse
import org.zxq.teleri.core.net.exception.ErrorResponseException
import org.zxq.teleri.core.utils.Json
import org.zxq.teleri.core.utils.NetUtil
import org.zxq.teleri.core.utils.SPHelper
import org.zxq.teleri.executor.base.ZXQConfig
import org.zxq.teleri.executor.base.ZXQParams
import org.zxq.teleri.executor.extensions.checkKey
import org.zxq.teleri.functionmodel.datacirculation.MsgCirculation
import org.zxq.teleri.homepage.bean.VehicleProfileBean
import org.zxq.teleri.homepage.cache.HomeCache
import org.zxq.teleri.homepage.cardetail.CarControlFragment
import org.zxq.teleri.mainentrance.MainPresenter
import org.zxq.teleri.mc.db.MessageCenterDbHelper
import org.zxq.teleri.model.bean.Cms4bBean
import org.zxq.teleri.personalcenter.PCHelper
import org.zxq.teleri.personalcenter.PersonalCenterFragment
import com.ebanma.sdk.core.domain.DomainUtil
import io.reactivex.functions.Consumer
import org.zxq.teleri.secure.utils.SPUtils
import org.zxq.teleri.ui.utils.AppUtils
import org.zxq.teleri.ui.utils.StringUtils
import org.zxq.teleri.ui.utils.UserRelationManager
import org.zxq.teleri.utils.VehicleFunctionUtils
import java.util.concurrent.TimeUnit

/**
 *Created by cyc on 2019/3/12.
 */

class CarViewModel(private val repository: CarRepository) : ViewModel() {


    companion object {
        fun instance(activity: FragmentActivity, repo: CarRepository): CarViewModel =
                ViewModelProvider(activity.viewModelStore, object : ViewModelProvider.Factory {
                    override fun <T : ViewModel> create(modelClass: Class<T>): T {
                        return CarViewModel(repo) as T
                    }
                }).get(CarViewModel::class.java)

        const val HTTP_ERROR = "ERROR"//标记单个请求失败
        const val HTTP_COMPLETE = "Complete"//标记主分支流程请求结束
        const val DATA_AIR_TEM_UP = "data_air_tem_up"//获取云端保存温度
        const val DATA_AIR_TEM_DOWN = "data_air_tem_down"//获取云端保存温度
    }

    //存储首页接口数据
    private val homeCache: HomeCache by lazy { MainPresenter.getInstance().homeCache }

    // 车辆权限适配表
    val vehicleFunctionBean = MutableLiveData<VehicleFunctionBean>()

    // 车辆数据信息
    var vehicleInfoBean = MutableLiveData<VehicleInfoAReqBean>()

    //车辆是否被盗
    val isVehicleStolen = MutableLiveData<Boolean>()

    //车辆状态
    var carStatus = MutableLiveData<CarStatus>()

    //车辆保养状态
    val carMaintainStatus = MutableLiveData<CarPartsStatusBean>()

    // 是否刷新完成
    val isRefreshComplete = MutableLiveData<Boolean>()

    val mBoundVinList = MutableLiveData<List<VinInfoBean>>()



    /**
     * 开始全部接口请求流程 [isRefresh]表示是否为刷新流程，非初始化流程
     */
    fun buildRequest() {

        val vinListObservable = Observable.just(PCHelper.getVinListCache()).subscribeOn(Schedulers.io())
                .flatMap { vinListBean ->
                    MainPresenter.getInstance().notifyObserverID(PersonalCenterFragment.KEY_PERSONAL_CENTER_FRAGMENT, PersonalCenterFragment.KEY_VINLIST_FROM_SERVER, true)

                    MainPresenter.getInstance().mBoundVinListBean = vinListBean
                    mBoundVinList.postValue(vinListBean)

                    if (!StringUtils.isEmpty(ZXQParams.getVin())) {
                        MsgCirculation.getInstance().cirCulaDataing()
                    }

                    MainPresenter.getInstance().dealSecondCar(vinListBean)
                    MainPresenter.getInstance().dealVinList()

                    vinListBean?.let { list ->
                        if (list.isNullOrEmpty()) {
                            EventBus.getDefault().post(RefreshVinFromServerEvent(false))
                            finishRequest()
                        } else {
                            getVpProfile()
                            getCmsListRequest()

                            if(!TextUtils.isEmpty(Framework.getAccountInject().vin)) {
                                requestVehicleStatus()
                            }
                            getFunCode()
                        }
                    }
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    finishRequest()
                }
    }

    //车型权限适配表下载
    private fun getFunCode(): Observable<String> = repository.getFunctionCode().subscribeOn(Schedulers.io())
            .onErrorReturn { throwable: Throwable ->
                checkResErrorWithToast(null, throwable)
                HTTP_ERROR
            }
            .flatMap { funCodeString ->
                if (HTTP_ERROR == funCodeString) {
                    EventBus.getDefault().post(RefreshVinFromServerEvent(false))
                    finishRequest()
                } else {
                    EventBus.getDefault().post(RefreshVinFromServerEvent(true))
                    val resultBean = Json.from(funCodeString, VehicleFunctionBean::class.java)
                    /*    resultBean.result?.forEach {
                            resultBean.functionMap[it.featureCode] = true
                        }*/

                    resultBean.isCache = false
                    VehicleFunctionUtils.setVehicleFunctionBean(resultBean)
                    MainPresenter.getInstance().vehicleFunctionBean = resultBean
                    homeCache.setCurrentVin(ZXQParams.getVin())
                    homeCache.saveHttpFile(HomeCache.HTTP_CACH_FILE_FUNCCODE, funCodeString)

                    //空调可以调温，需要获取云端配置
                    if (resultBean.functionMap.checkKey(FunctionCode.C_2_1_4_1_0)) {
                        getTemSaveMessageRequest()
                    }
                    //不是授权用户 要请求被盗接口
                    if (!UserRelationManager.isUserGrant() && !StringUtils.isEmpty(ZXQParams.getVin())) {
                        getCarStolenRequest()
                    }
                    //获取车辆保养状态
                    if (resultBean.functionMap.checkKey(FunctionCode.C_1_1_2_1_0) || UserRelationManager.isUserNormal()) {
                        getCarMaintainRequest()
                        MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT,
                                CarControlFragment.KEY_CAR_RESET_NEED_MAINTAIN, -1)
                    }

                    MainPresenter.getInstance().notifyObserverID(PersonalCenterFragment.KEY_PERSONAL_CENTER_FRAGMENT, PersonalCenterFragment.KEY_GET_FUNCTION_CODE)

                    if (MainPresenter.getInstance().isVinChange) {
                        MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT, CarControlFragment.KEY_CAR_REFRESH_FIRST)
                    } else {
                        MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT, CarControlFragment.KEY_CAR_REFRESH)
                    }

                    vehicleFunctionBean.postValue(resultBean)
                    if (!StringUtils.isEmpty(ZXQParams.getVin())) {
                        getZipResult()
                    } else {
                        Observable.just(HTTP_COMPLETE)
                    }
                }
            }


    /**
     * 获取车控配置 ，运行时间
     */
    private fun getVpProfile() = repository.getVpProfile().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .doOnNext { jsonString ->
                if (HTTP_ERROR != jsonString) {
                    val bean = Json.from(jsonString, VehicleProfileBean.DataBean::class.java)
                    bean?.let {
                        with(it.app_timeout) {
                            if (this > 0) {
                                ZXQConfig.setExecuteTimeOut(this * 1000)
                            }
                        }
                        with(it.engine_close) {
                            if (this > 0) {
                                ZXQConfig.setBusActiveAliveTime(this * 1000)
                            }
                        }
                        with(it.event_time_out) {
                            if (this > 0) {
                                ZXQConfig.setEventAliveTime(this / 60)
                            }
                        }
                    }
                }
            }
            .subscribe()

    /**
     * 请求车辆状态上报
     */
    private fun requestVehicleStatus() = repository.requestVehicleStatus().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .subscribe()

    /**
     * 活动的未读消息需要接口支持 其余读取本地数据库监听 未读消息初始监听
     */
    private fun getCmsListRequest() = repository.getCmsListRequest().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .doOnNext {
                if (HTTP_ERROR != it) {
                    val data = Json.fromList(it, Cms4bBean::class.java) as List<Cms4bBean>
                    if (!data.isNullOrEmpty()) {
                        val allActivityWebUrls = MessageCenterDbHelper.queryAllActiveWebFromDb()
                        for (activity in data) {
                            if (allActivityWebUrls.contains(activity.activity_id)) {
                                break
                            }
                            MessageCenterDbHelper.writeMessageActiveDb(activity)
                        }
                    } else run { MessageCenterDbHelper.updateReadActive() }
                }
            }.subscribe()


    /**
     * 获取车辆状态
     */
    fun getCarStatusRequest(): Observable<String> = repository.getVehicleStateData().subscribeOn(Schedulers.io())
            .onErrorReturn {
                checkResErrorWithToast(null, it)
                HTTP_ERROR
            }
            .doOnNext { responseString ->
                if (HTTP_ERROR == responseString) {
                    SPUtils.putInt(SPUtils.XML_CACHE, SPUtils.ODOMETER, -1)
                    finishRequest()
                } else {
                    val carStatusBean = Json.from<CarStatus>(responseString, CarStatus::class.java)
                    MainPresenter.getInstance().compleData(carStatusBean)

//                    updateCarStatusSubject(carStatusBean)

                    val vehiclePosition = carStatusBean.getVehiclePosition()
                    vehiclePosition?.let { position ->
                        if (position.latitude == 0.0 && position.longitude == 0.0) {
                            val jsonStr = SPHelper.getString(SPHelper.HTTP_CACHE_CAR_POSITION
                                    + Framework.getAccountInject().vin)
                            val pos = Json.from(jsonStr, Position::class.java)
                            if (pos?.latitude != 0.0 && pos?.longitude != 0.0) {
                                carStatusBean.setVehiclePosition(pos)
                            }
                        }
                        val saveJsonStr = Json.to(position)
                        SPHelper.setPreference(SPHelper.HTTP_CACHE_CAR_POSITION + Framework.getAccountInject().vin,
                                saveJsonStr)
                    }
                    val vehicleValue = carStatusBean.getVehicleValue()
                    vehicleValue?.let {
                        SPUtils.putInt(SPUtils.XML_CACHE, SPUtils.ODOMETER, it.odometer)
                    }

                    MainPresenter.getInstance().carStatus = carStatusBean

                    carStatus.postValue(carStatusBean)

                    //添加缓存
                    MainPresenter.getInstance().homeCache.saveHttpFile(HomeCache.HTTP_CACH_FILE_CARSTATUS, Json.to(carStatusBean))


                    //列表不为空，通知车控状态查询
                    if (MainPresenter.getInstance().isHaveVin) {
                        MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT, CarControlFragment.KEY_CAR_REFRESH_CAR_CONTRIL)
                        MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT, CarControlFragment.KEY_CAR_STATUS_REFRESH)
                    }
                }
            }


    /**
     *请求用户蓝牙钥匙列表
     */
    private fun getBluetoothKeyList(): Observable<String> = repository.getBlueKeyList().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .doOnNext { result ->
                if (HTTP_ERROR != result) {
                    val listUserBluetoothKeyBean = Json.from(result, ListBluetoothKeyBean::class.java)
                    val datas = listUserBluetoothKeyBean.data
                    datas?.let {
                        SSUtils.putString(SPUtils.ZXQ_BLE, "blekey_" + ZXQParams.getUserId(), result)
                        val bleVin = datas.vin
                        if (ZXQParams.getVin() == bleVin) {
                            val keyPrivilege = datas.key_privilege
                            when (keyPrivilege) {
                                1 -> {//  //只有开、关门
                                    MainPresenter.getInstance().vehicleFunctionBean.addCode(FunctionCode.C_1_3_1_1_0)
                                    MainPresenter.getInstance().vehicleFunctionBean.removeCode(FunctionCode.C_1_3_1_2_0)
                                }
                                2 -> {//只有后备箱
                                    MainPresenter.getInstance().vehicleFunctionBean.removeCode(FunctionCode.C_1_3_1_1_0)
                                    MainPresenter.getInstance().vehicleFunctionBean.addCode(FunctionCode.C_1_3_1_2_0)
                                }
                                3 -> {//全权限
                                    MainPresenter.getInstance().vehicleFunctionBean.addCode(FunctionCode.C_1_3_1_1_0)
                                    MainPresenter.getInstance().vehicleFunctionBean.addCode(FunctionCode.C_1_3_1_2_0)
                                }
                            }

                            VehicleFunctionUtils.setVehicleFunctionBean(MainPresenter.getInstance().vehicleFunctionBean)

                            MainPresenter.getInstance().vehicleFunctionBean.isCache = false

                            SPUtils.putString(SPUtils.XML_CONFIG, UserLogin.getAccountA().id.toString() + DomainUtil.BizType.user.domainUrl, Json.to(vehicleFunctionBean.value))
                            MainPresenter.getInstance().notifyObserverID(CarControlFragment.KEY_CARCONTROL_FRAGMENT, CarControlFragment.KEY_CAR_BLUE_LIST)

                            //更新vehicleFunctionBean的值
                            vehicleFunctionBean.postValue(MainPresenter.getInstance().vehicleFunctionBean)
                        }
                    }
                }
            }

    /**
     *  请求车辆信息（绑定 激活相关）
     */
    private fun getCarInfo(): Observable<String> = repository.getCarInfo().subscribeOn(Schedulers.io())
            .onErrorReturn {
                checkResErrorWithToast(null, it)
                HTTP_ERROR
            }
            .doOnNext { result ->
                if (HTTP_ERROR != result) {
                    val vehicleInfo = Json.from<VehicleInfoAReqBean>(result, VehicleInfoAReqBean::class.java)

                    vehicleInfo?.let {

                        vehicleInfo.isCache = false

                        MainPresenter.getInstance().vehicleInfoBean = vehicleInfo

                        vehicleInfo.vehicleProjectCode?.let {
                            Framework.getDataRecord().addVehicleMode(it)
                        }

                        vehicleInfoBean.postValue(vehicleInfo)

                        MainPresenter.getInstance().notifyObserverID(MyCarListActivity.OBSERVER_KEY_ID,
                                MyCarListActivity.KEY_GET_CAR_INFO_DATA, true)

                        MainPresenter.getInstance().notifyObserverID(PersonalCenterFragment.KEY_PERSONAL_CENTER_FRAGMENT, PersonalCenterFragment.KEY_FINDCARINFO_FROM_SERVER)

                        //添加缓存
                        MainPresenter.getInstance().homeCache.saveHttpFile(HomeCache.HTTP_CACH_FILE_CARINFO, Json.to(vehicleInfo))
                    }
                }
            }


    /**
     *  车辆状态，蓝牙钥匙、车辆激活等信息接口全部结束后刷新
     */
    private fun getZipResult(): Observable<String> = Observable.zip(getCarStatusRequest(), getBluetoothKeyList(), getCarInfo(),
            Function3<String, String, String, String> { _, _, _ ->
                HTTP_COMPLETE
            })


    /**
     * 获取云端保存温度
     */
    private fun getTemSaveMessageRequest() = repository.getTemSaveMessageRequest().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .doOnNext { result ->
                if (HTTP_ERROR != result) {
                    val temCloudBean = Json.from(result, TemCloudBean::class.java)
                    val share = ZXQApplication.getContext().getSharedPreferences(SPUtils.XML_CONFIG,
                            Activity.MODE_PRIVATE)
                    with(share.edit()) {
                        putString(DATA_AIR_TEM_UP + ZXQParams.getVin(), temCloudBean.highTemp)
                        putString(DATA_AIR_TEM_DOWN + ZXQParams.getVin(), temCloudBean.lowTemp)
                        apply()
                    }
                }
            }
            .subscribe()


    /**
     * 获取车辆被盗报状态
     */
    fun getCarStolenRequest(): Disposable = repository.getCarStolenStatus().subscribeOn(Schedulers.io())
            .onErrorReturn {
                checkResErrorWithToast(null, it)
                HTTP_ERROR
            }
            .doOnNext { result ->
                if (HTTP_ERROR != result) {
                    val isStolen = Json.from(result, Boolean::class.java)
                    isVehicleStolen.postValue(isStolen)
                    //通知首页更新车辆被盗报警View显示
                    CarEvent.showCarDanger.onNext(isStolen)
                }

            }
            .subscribe()


    /**
     *  车辆保养接口相关信息
     */
    private fun getCarMaintainRequest() = repository.getCarMaintain().subscribeOn(Schedulers.io())
            .onErrorReturn { HTTP_ERROR }
            .doOnNext { result ->
                if (HTTP_ERROR != result) {
                    val bean = Json.from(result, CarPartsStatusBean::class.java)

                    MainPresenter.getInstance().updateTvRemind(bean)
                    carMaintainStatus.postValue(bean)
                }
            }
            .subscribe()


    /**
     * 结束刷新流程
     */
    private fun finishRequest(): Observable<String> {

        MainPresenter.getInstance().isCarControlReady()

        CarEvent.pullRefreshComplete.onNext(true)
        CarEvent.refreshMap.onNext(true)

        if (true != isRefreshComplete.value) {
            isRefreshComplete.postValue(true)
        }
        return Observable.empty<String>()
    }


    private fun checkResErrorWithToast(result: String?, e: Throwable?): Boolean {
        return if (e != null || result == null) {
            when (e) {
                is NetUtil.NetErrorException -> AppUtils.showToast(R.string.network_disconnected)

                is ErrorResponseException -> OnErrorResponse.getInstance().accept(e)
                else -> AppUtils.showToast(R.string.network_disconnected)
            }
            false
        } else {
            true
        }
    }

}
