package org.zxq.teleri.homepage.cardetail.viewinter

import android.Manifest
import android.app.Activity
import android.arch.lifecycle.Observer
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.provider.Settings
import android.support.v4.app.FragmentActivity
import android.support.v7.widget.LinearLayoutManager
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewTreeObserver
import com.amap.api.maps.AMap
import com.amap.api.maps.AMapOptions
import com.amap.api.maps.CameraUpdateFactory
import com.amap.api.maps.model.LatLng
import com.amap.api.services.core.LatLonPoint
import com.timehop.stickyheadersrecyclerview.StickyRecyclerHeadersDecoration
import io.reactivex.android.schedulers.AndroidSchedulers
import kotlinx.android.synthetic.main.fragment_home_page.*
import org.zxq.teleri.R
import org.zxq.teleri.amap.AMapUtil
import org.zxq.teleri.bean.LatLonPointBean
import org.zxq.teleri.common.view.pageswitch.HomePageSwitcher
import org.zxq.teleri.common.view.pageswitch.PageSwitcher
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.base.SuperFragment
import org.zxq.teleri.core.constant.FunctionCode
import org.zxq.teleri.core.constant.RouteUrl
import org.zxq.teleri.core.event.Event
import org.zxq.teleri.core.route.Router
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.core.utils.NetUtil
import org.zxq.teleri.executor.base.BasePresenterImpl
import org.zxq.teleri.executor.extensions.checkKey
import org.zxq.teleri.global.Constant
import org.zxq.teleri.homepage.cardetail.CarControlFragment
import org.zxq.teleri.journeyarrange.JourneyArrangeAdapter
import org.zxq.teleri.journeyarrange.JourneyArrangeBean
import org.zxq.teleri.journeyarrange.JourneyArrangeContract
import org.zxq.teleri.journeyarrange.JourneyArrangePresenter
import org.zxq.teleri.mainentrance.MainPresenter
import org.zxq.teleri.mc.utils.MsgPushHelper
import org.zxq.teleri.scan.ScanCameraGrantActivity
import org.zxq.teleri.searchpoi.SearchPoiActivity
import org.zxq.teleri.security.common.commontool.UITools
import org.zxq.teleri.security.helper.PinCodeHelper
import org.zxq.teleri.ui.permission.CommonDialogPermissionUtil
import org.zxq.teleri.ui.permission.PermissionBackListener
import org.zxq.teleri.ui.theme.ThemeManager
import org.zxq.teleri.ui.utils.*
import org.zxq.teleri.utils.*
import org.zxq.teleri.vehicleposition.VehiclePositionActivity
import org.zxq.teleri.vehicleposition.VehiclePositionPresenter
import java.util.*
import kotlinx.android.synthetic.main.fragment_home_page.rv_journey as mJourneyRecyclerView


/**
 *Created by cyc on 2019/3/7.
 */
class HomeFragment : SuperFragment(), View.OnClickListener,
        JourneyArrangeContract.View, HomePageSwitcher.JourneyArrangePullListener  {

    private lateinit var journeyDecoration: StickyRecyclerHeadersDecoration

    //行程列表相关
    private lateinit var journeyArrangeAdapter: JourneyArrangeAdapter
    private lateinit var mTodayJourneyList: MutableList<JourneyArrangeBean>
    private lateinit var journeyArrangeData: MutableList<JourneyArrangeBean>
    private lateinit var mJourneyDataList: MutableList<JourneyArrangeBean>


    private lateinit var mAMap: AMap

    private lateinit var carFragment: CarControlFragment

    private lateinit var mJourneyArrangePresenter: JourneyArrangeContract.Presenter

    private lateinit var mLayoutManager: LinearLayoutManager

    private var isLoad = false
    private var isBindStatus = false

    private var statusHeight = 0
    private var currentTime = 0L

    private var mCarSecurityPresenter: BasePresenterImpl? = null

    private lateinit var mVehiclePositionPresenter: VehiclePositionPresenter

    private var mMarkList: MutableList<LatLonPointBean>? = null

    private lateinit var carViewModel: CarViewModel

    private var isCreateView = false


    override fun onAttach(context: Context?) {
        super.onAttach(context)

        //主题theme订阅
        val themeDisposable = ThemeManager.ThemeSubject.observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    journeyDecoration.invalidateHeaders()
                    mJourneyRecyclerView.postInvalidate()
                }
        mCompositeDisposable.add(themeDisposable)

        //更新地图上的用户logo和位置
        val userInfoUpdateDisposable = Event.USER_INFO_UPDATE.subscribe { photoUrl ->
            //更换地图上用户头像
            mMarkList?.first {
                VehiclePositionUtils.TYPE_AVATAR == it.tag
            }?.setmPhotoUrl(photoUrl)
            updateLocationOnMapWrapper()
        }
        mCompositeDisposable.add(userInfoUpdateDisposable)

        registerEvent()
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mJourneyArrangePresenter = JourneyArrangePresenter(this)
        mVehiclePositionPresenter = VehiclePositionPresenter()
        mVehiclePositionPresenter.setCompositeDisposable(mCompositeDisposable)
        mVehiclePositionPresenter.setVehiclePositonListener(object : VehiclePositionPresenter.VehiclePositionListener {
            override fun getAllPositionDatas(mList: MutableList<LatLonPointBean>?, isShowDialog: Boolean) {
            }

            override fun getAllPositionExceptVehicle(mList: MutableList<LatLonPointBean>?, bean: LatLonPointBean?) {
                mMarkList = mList
                updateLocationOnMapWrapper()
            }

            override fun updateVehiclePosition(bean: LatLonPointBean?) {
                //车辆位置显示
                showVehicleStatus(bean)
            }

        })

        if(NetUtil.isNetWorkConnected()) {
            mJourneyArrangePresenter.start()
        }else{
            ToastUtil.show(R.string.network_disconnected)
            mJourneyArrangePresenter.loadJourney()
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val view = inflater.inflate(R.layout.fragment_home_page, container, false)
        isCreateView = true
        view.findViewById<View>(R.id.imv_icon_car_alert).setOnClickListener(this)
        view.findViewById<View>(R.id.view_position).setOnClickListener(this)
        view.findViewById<View>(R.id.fl_vehicle).setOnClickListener(this)
        view.findViewById<View>(R.id.ll_vehicle).setOnClickListener(this)
        return view
    }


    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        statusHeight = DisplayUtil.getStatusBarHeight(context)

        initViewModelObserve()

        initMap(savedInstanceState)

        setViewsPoi()

        initTitleBar()

        addCarFragment()

        initPageSwitcher()

        initJourneyRecyclerView()

    }

    private fun registerEvent() {

        //刷新首页地图数据缓存
        val refreshMapCacheDisposable = CarEvent.refreshMapCache.subscribe {
            mVehiclePositionPresenter.initCache()
        }
        mCompositeDisposable.add(refreshMapCacheDisposable)

        //通知地图刷新
        val refreshMapDisposable = CarEvent.refreshMap.subscribe {
            var flag = false
            val vehicleFunctionBean = VehicleFunctionUtils.getSingleVehicleFuncationInstance()
            val personCode = vehicleFunctionBean.functionMap.checkKey(FunctionCode.C_1_2_1_3_0)
            if (AMapUtil.isLocationService() && (personCode || UserRelationManager.isUserNormal())) {
                flag = true
            }
            mVehiclePositionPresenter.initDatas(0, flag, true)
        }
        mCompositeDisposable.add(refreshMapDisposable)

        //下拉刷新结束
        val refreshCompleteDisposable = CarEvent.pullRefreshComplete.subscribe {
            viewRefFinish()
        }
        mCompositeDisposable.add(refreshCompleteDisposable)

        //显示车辆被盗报警View
        val showCarDangerDisposable = CarEvent.showCarDanger.subscribe {
                view!!.post() {
                    imv_icon_car_alert?.setVisibility(if (it == true) View.VISIBLE else View.GONE)
                }
        }
        mCompositeDisposable.add(showCarDangerDisposable)

        val back2FirstPageDisposable = CarEvent.back2FirstPage.subscribe {
            if(pageSwitcher!=null) {
                pageSwitcher.backToFirstScreen()
            }else{
                LogUtils.i("Andy","pageSwitcher == null")
            }
        }
        mCompositeDisposable.add(back2FirstPageDisposable)

    }

    private fun updateLocationOnMapWrapper() {
        if (activity == null) {
            return
        }
        val permissions = ArrayList<String>()
        permissions.add(Manifest.permission.ACCESS_FINE_LOCATION)
        CommonDialogPermissionUtil.checkMultPermission(activity, permissions,
                object : PermissionBackListener {
                    override fun actionAllSus() {
                        updateLocationOnMap()
                    }

                    override fun actionAllHasFailCancle() {
                    }

                    override fun actionAllHasFailOnce(activity: Activity, listener: PermissionBackListener.BackListener) {
                    }

                    override fun actionAllHasFailNever(activity: Activity) {
                    }
                })
    }

   private fun updateLocationOnMap() {
        if (!isLoad) {
            return
        }
        val pointY = if (UserRelationManager.isUserNormal()) {
            DisplayUtil.dip2px(256 - 44) - statusHeight
        } else {
            DisplayUtil.dip2px(256 - 44 - 40) - statusHeight
        }
        //设置屏幕上的某个像素点为地图中心点
        val centerY = pointY / 2 + DisplayUtil.dip2px(44) + statusHeight + DisplayUtil.dip2px(28)
        val yUp = DisplayUtil.dip2px(78 + 92 * 7 / 9)
        val yFloor = DisplayUtil.dip2px(44 + 92 * 2 / 9) + pointY + statusHeight
        mAMap.setPointToCenter(DisplayUtil.getDisplayWidth() / 2, centerY)
        //判断有无用户icon
        val markerListByTag = VehiclePositionUtils.getInstance().getMarkerListByTag(VehiclePositionUtils.TYPE_AVATAR, mMarkList)
        var center: LatLonPointBean? = null
        markerListByTag?.let {
            if (it.isNotEmpty()) {
                center = it[0]
                if (center?.getmPoint() != null) {
                    mAMap.moveCamera(CameraUpdateFactory.changeLatLng(AMapUtil.convertToLatLng(center?.getmPoint())))
                }
            }
        }
        mAMap.clear()
        mMarkList?.forEach {
            val marOptions = VehiclePositionUtils.getInstance().getMarOptions(it.tag, it)
            val marker = mAMap.addMarker(marOptions)
            it.marker = marker
            marker.isClickable = false
            VehiclePositionUtils.getInstance().changeIcon(it, mCompositeDisposable)
        }
        if (mMarkList.isNullOrEmpty() || mMarkList?.size == 1) {
            mAMap.moveCamera(CameraUpdateFactory.zoomTo(mAMap.maxZoomLevel))
        } else {
            //在地图的可视范围内显示
            if (centerY in yUp..yFloor) {
                val space = Math.min(centerY - yUp, yFloor - centerY)
                val latLngBounds = VehiclePositionUtils.getInstance()
                        .getLatLngBounds(if (center == null) null else AMapUtil.convertToLatLng(center?.getmPoint()), mMarkList)
                mAMap.animateCamera(CameraUpdateFactory.newLatLngBoundsRect(latLngBounds,
                        DisplayUtil.dip2px(50), DisplayUtil.dip2px(50), centerY - space, DisplayUtil.dip2px(410) - centerY - space))
            }
        }
    }

    override fun onPause() {
        mapView?.onPause()
        Framework.getDataRecord().leavePage("home", null)
        super.onPause()
    }

    override fun onResume() {
        super.onResume()
        mapView?.onResume()
        Framework.getDataRecord().enterPage("home", null)
    }

    override fun onDestroyView() {
        MsgPushHelper.instance.unRegister()
        super.onDestroyView()
    }

    override fun onDestroy() {
        mapView?.onDestroy()
        mCarSecurityPresenter?.releaseMemory()
        super.onDestroy()
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.imv_icon_car_alert, R.id.view_position, R.id.fl_vehicle -> verifyPinCode(true)
            R.id.ll_vehicle -> verifyPinCode(false)
        }
    }

    private fun gotoMapActivity(flag: Boolean) {
        Framework.getDataRecord().ctrlClicked("home", "location_people")
        val intent = Intent(activity, VehiclePositionActivity::class.java)
        intent.apply {
            addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT)
            putExtra(VehiclePositionUtils.KEY_NAME, flag)
            ActivitySplitAnimationUtil.prepare(activity, DisplayUtil.dip2px(78))
            startActivityForResult(this, Constant.REQUEST_CODE)
            activity?.overridePendingTransition(0, 0)
        }
    }

    override fun isPageReady2Pull() = mLayoutManager.findFirstCompletelyVisibleItemPosition() == 0

    override fun setUserVisibleHint(isVisibleToUser: Boolean) {
        super.setUserVisibleHint(isVisibleToUser)
        if (::carFragment.isInitialized) {
            carFragment.userVisibleHint = isVisibleToUser
        }
    }

    override fun reload4switch() {
        if (NetUtil.isNetWorkConnected()) {
            if (isCreateView) {
                view_loading.visibility = View.VISIBLE
                pageSwitcher.setRefreshing(true)
                MainPresenter.getInstance().notifyHomeRefreshing(true)
                /**
                 * 发布首页下拉刷新事件
                 */
                Event.HOME_PTR.onNext("")

                MainPresenter.getInstance().reFreshMain()
                mJourneyArrangePresenter.loadJourney()
            }
        } else {
            ToastUtil.show(R.string.network_disconnected)
        }
    }

    private fun isCarAlerting(): Boolean {
        val data = carViewModel.vehicleInfoBean.value
        val isBind = data?.let {
            it.vehicleBindStatus == 3
        } ?: false

        val isVehicleStolen = carViewModel.isVehicleStolen.value ?: false
        if (isVehicleStolen && isBind) {
            return true
        }
        val carStatus = carViewModel.carStatus.value
        val vehicleFunctionBean = carViewModel.vehicleFunctionBean.value
        var isAlerting = false
        if (vehicleFunctionBean?.functionMap?.checkKey(FunctionCode.C_2_3_2_0_0) == true) {
            carStatus?.let { status ->
                val vehicleAlerts = status.getVehicleAlerts()
                if (vehicleAlerts.any { 2 == it.alertType }) {
                    isAlerting = isBind
                }
            }
        }
        return isAlerting
    }


    private fun hideGaodeButton() {
        val settings = mAMap.uiSettings
        settings?.apply {
            isZoomControlsEnabled = false
            // 设置默认定位按钮是否显示
            isMyLocationButtonEnabled = false
            logoPosition = AMapOptions.LOGO_POSITION_BOTTOM_CENTER//设置logo位置
            setLogoBottomMargin(-500)
            setAllGesturesEnabled(false)
        }
    }

    override fun setPresenter(presenter: JourneyArrangeContract.Presenter) {
        mJourneyArrangePresenter = presenter
    }

    override fun showJourney(data: MutableList<JourneyArrangeBean>, isCache: Boolean, gpsStatus: Boolean) {
        mJourneyDataList.clear()
        mTodayJourneyList.clear()

        mJourneyDataList = data
        mJourneyDataList.forEach {
            if (DateUtils.isToday(it.datetime)
                    || it.viewType == JourneyArrangeAdapter.VIEW_TYPE_HIDE
                    || it.viewType == JourneyArrangeAdapter.VIEW_TYPE_SHOW
                    || it.viewType == JourneyArrangeAdapter.VIEW_TYPE_FOOTER
                    || it.viewType == JourneyArrangeAdapter.VIEW_TYPE_NO_JOURNEY
                    || it.viewType == JourneyArrangeAdapter.VIEW_TYPE_NO_NETWORK) {
                if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_HIDE) {
                    it.isNeedShow = false
                } else if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_SHOW) {
                    it.isNeedShow = true
                }
                mTodayJourneyList.add(it)
            }
        }

        if (!isAdded) {
            return
        }

        activity?.runOnUiThread {
            LogUtils.d("setData==invalidateHeaders")
            //默认只显示当天行程
            journeyDecoration.invalidateHeaders()
            journeyArrangeAdapter.replaceData(mTodayJourneyList, isCache, gpsStatus)

            journeyArrangeAdapter.setOnTextClickListener(object : JourneyArrangeAdapter.OnTextClickListener {
                override fun onShowClick() {

                    mJourneyDataList.forEach {
                        if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_HIDE) {
                            it.isNeedShow = true
                        } else if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_SHOW) {
                            it.isNeedShow = false
                        }
                    }
                    //显示
                    journeyArrangeAdapter.replaceData(mJourneyDataList, isCache, gpsStatus)
                    Framework.getDataRecord().ctrlClicked("home_routelist", "routelist_other")
                }

                override fun onHideClick() {
                    mTodayJourneyList.forEach {
                        if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_HIDE) {
                            it.isNeedShow = false
                        } else if (it.viewType == JourneyArrangeAdapter.VIEW_TYPE_SHOW) {
                            it.isNeedShow = true
                        }
                    }
                    journeyArrangeAdapter.replaceData(mTodayJourneyList, isCache, gpsStatus)
                }

            })
            journeyArrangeAdapter.setOnItemClickListener { _, position ->
                if (AppUtils.isFastDoubleClick()) {
                    return@setOnItemClickListener
                }
                journeyArrangeAdapter.data?.let {
                    Framework.getDataRecord().ctrlClicked("home_routelist", "routelist_activity")
                    LogUtils.w("url=" + journeyArrangeAdapter.data[position].detailUrl)
                    IntentUtil.startWebActivity(activity,
                            it[position].detailUrl)

                }
            }
        }
    }

    override fun openGPSSetting() {
        val intent = Intent()
        intent.let {
            it.action = Settings.ACTION_LOCATION_SOURCE_SETTINGS
            try {
                startActivityForResult(it, Constant.RESULT_GPS_CODE)
            } catch (ex: ActivityNotFoundException) {
                it.action = Settings.ACTION_SETTINGS
                try {
                    startActivityForResult(it, Constant.RESULT_GPS_CODE)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
    }

    override fun showAd() {}

    override fun isActive() = isAdded

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == Constant.REQUEST_CODE && resultCode == Constant.RESULT_CODE) {
            frame_bottom.visibility = View.VISIBLE
            linear_search.visibility = View.VISIBLE
            frame_bottom.animation = UIUtils.movetoViewLocationFromBottom()
            linear_search.animation = UIUtils.movetoViewLocationFromTop()
        } else if (requestCode == Constant.RESULT_GPS_CODE) {
            journeyArrangeAdapter.setGpsStatus(AMapUtil.isOpenGps())
        }
    }

    /**
     * @param flag true全览模式
     */
    private fun verifyPinCode(flag: Boolean) {
        if (System.currentTimeMillis() - currentTime < 1000) {
            return
        }
        currentTime = System.currentTimeMillis()
        if (UserRelationManager.isUserNormal()) {
            //普通用户
            gotoMapActivity(flag)
            return
        }
        mCarSecurityPresenter = BasePresenterImpl()
        val mP = PinCodeHelper(activity, mCarSecurityPresenter)
        mP.checkPinCode { result ->
            LogUtils.d("result : $result")
            //安防
            if (isCarAlerting()) {
                Router.route(RouteUrl.URL_MAIN_CAR_SECURE)
            } else {
                if (UserRelationManager.isUserOwner() || UserRelationManager.isUserGrant()
                        || UserRelationManager.isUserSecondCar()) {
                    //位置允许授权用户查看，当前账号在车机登录时可用
                    gotoMapActivity(flag)
                }
            }
        }
    }

    private fun setViewsPoi() {
        val params = frame_bottom.layoutParams as ViewGroup.MarginLayoutParams
        params.topMargin = statusHeight
        frame_bottom.layoutParams = params
        val paramsPoi = view_position.layoutParams as ViewGroup.MarginLayoutParams
        paramsPoi.topMargin = statusHeight
        view_position.layoutParams = paramsPoi
        linear_search.setPadding(0, statusHeight, 0, 0)
        val paramsLoading = viewAnimLoading.layoutParams as ViewGroup.MarginLayoutParams
        paramsLoading.topMargin = UIUtils.dip2px((44 + 15).toFloat()) + statusHeight
        viewAnimLoading.layoutParams = paramsLoading
        val paramsDanger = imv_icon_car_alert.layoutParams as ViewGroup.MarginLayoutParams
        val pointY = UIUtils.dip2px((256 - 44 - 40).toFloat()) - statusHeight
        val centerY = pointY / 2 + UIUtils.dip2px(44f) + UIUtils.dip2px(28f)
        val marginTop = centerY - UIUtils.dip2px(75f)
        paramsDanger.topMargin = marginTop
        imv_icon_car_alert.layoutParams = paramsDanger
    }

    private fun showVehicleStatus(data: LatLonPointBean?) {
        if (data?.getmPoint() == null || (data.getmPoint().longitude == 0.0 && data.getmPoint().latitude == 0.0)) {
            fl_vehicle.visibility = View.GONE
            return
        }
        val latLonPoint = data.getmPoint()
        val vehicleFunctionBean = carViewModel.vehicleFunctionBean.value
        val carCode = vehicleFunctionBean?.functionMap?.checkKey(FunctionCode.C_1_2_1_1_0) ?: false
        val dataBean = carViewModel.vehicleInfoBean.value
        dataBean?.let {
            isBindStatus = it.vehicleBindStatus == 3
        }
        if (isBindStatus && carCode && latLonPoint != null && (latLonPoint.latitude != 0.0 || latLonPoint.longitude != 0.0)) {
            val point = LatLonPoint(latLonPoint.latitude, latLonPoint.longitude)
            val subscribe = VehiclePositionUtils.getInstance().getGeocoderSearchAddress(point, 1, true).doOnNext { bean ->
                var vehicleBuffer = StringBuilder(bean.getmAddress())
                if (bean.getmAddress().length > 7) {
                    vehicleBuffer = vehicleBuffer.replace(7, bean.getmAddress().length, "...")
                }
                tv_vehicle_address.text = vehicleBuffer.toString()
                fl_vehicle.visibility = View.VISIBLE
            }.subscribe()
            mCompositeDisposable.add(subscribe)
        } else {
            fl_vehicle.visibility = View.GONE
        }
    }

    private fun initJourneyRecyclerView() {
        mLayoutManager = LinearLayoutManager(context)
        mJourneyRecyclerView.layoutManager = mLayoutManager
        mTodayJourneyList = mutableListOf()
        mJourneyDataList = mutableListOf()
        journeyArrangeData = mutableListOf()
        journeyArrangeAdapter = JourneyArrangeAdapter(journeyArrangeData)
        mJourneyRecyclerView.adapter = journeyArrangeAdapter
        journeyDecoration = StickyRecyclerHeadersDecoration(journeyArrangeAdapter)
        mJourneyRecyclerView.addItemDecoration(journeyDecoration)
    }

    //ViewModel中的LiveData数据监听处理
    private fun initViewModelObserve() {
        carViewModel = ViewModelInjectUtil.getCarViewModel(activity as FragmentActivity)


        //监听下拉刷新结束
        val refreshLoadingObserver = Observer<Boolean> {
            if (true == it) {
                if (view_loading.visibility == View.VISIBLE) {
                    view_loading.visibility = View.GONE
                }
            }
        }
        carViewModel.isRefreshComplete.observe(this, refreshLoadingObserver)

    }

    private fun initPageSwitcher() {
        pageSwitcher.addBuildCfg(PageSwitcher.BuildCfg()
                .setNoEventSpace(DisplayUtil.dip2px(250.0)).setPage2MarginTop(DisplayUtil.dip2px(48.0)))

        pageSwitcher.setPullEventLister(object : PageSwitcher.PullEventLister {
            override fun onScrollPosition(scrolly: Int) {
                if (scrolly <= 0) {
                    viewAnimLoading.scrollTo(0, -scrolly)
                    if (Math.abs(scrolly) >= pageSwitcher.page1Height) {
                        carFragment.view?.alpha = 0f
                    } else {
                        carFragment.view?.alpha = 1f
                    }
                    fl_vehicle.scrollY = Math.abs(scrolly)
                } else {
                    fl_vehicle.scrollY = 0
                }
            }

            override fun onRefresh() {
                if(NetUtil.isNetWorkConnected()) {
                    view_loading.visibility = View.VISIBLE
                    pageSwitcher.setRefreshing(true)
                    MainPresenter.getInstance().notifyHomeRefreshing(true)
                    /**
                     * 发布首页下拉刷新事件
                     */
                    Event.HOME_PTR.onNext("")
                }else{
                    ToastUtil.show(R.string.network_disconnected)
                }
            }

            override fun onRefreshViewReset() {
                MainPresenter.getInstance().reFreshMain()
                mJourneyArrangePresenter.loadJourney()
            }

        })

        pageSwitcher.setJourneyArrangePullListener(this)

        pageSwitcher.addPageListener {
            if (it == 1) {
                Framework.getDataRecord().enterPage("home_routelist")
            }
        }

    }

    private fun initTitleBar() {
        imv_icon_car_alert.setTouchable(false)

        viewSearchBar.setOnClickListener {
            if (AppUtils.isFastDoubleClick()) {
                return@setOnClickListener
            }
            Framework.getDataRecord().ctrlClicked("poifeature", "input", Framework.getDataRecord().buildManager().type("home").build())
            startActivity(Intent(activity, SearchPoiActivity::class.java))
        }

        UITools.clikAlpha(viewTitleBarMessage) {
            Router.route(RouteUrl.URL_MESSAGE_CENTER, "messageCommon")
        }

        MsgPushHelper.instance.addMultiMsgBackListener(object : MsgPushHelper.BackListener {
            override fun onUnReadMsg(numb: Int) {
                if (numb <= 0) {
                    msgNumbView.visibility = View.GONE
                } else {
                    msgNumbView.visibility = View.VISIBLE
                    msgNumbView.setText(if (numb > 99) "..." else numb.toString())
                }
            }
        })

        viewScan.setOnClickListener {
            if (!AppUtils.isFastDoubleClick()) {
                startActivity(Intent(context, ScanCameraGrantActivity::class.java))
            }
        }
    }

    private fun addCarFragment() {
        fragment_container.viewTreeObserver.addOnGlobalLayoutListener(
                object : ViewTreeObserver.OnGlobalLayoutListener {
                    override fun onGlobalLayout() {
                        if (!isAdded) {
                            return
                        }
                        val outRect = Rect()
                        activity?.window?.decorView?.getWindowVisibleDisplayFrame(outRect)
                        if (outRect.right == 0 || outRect.bottom == 0) {
                            return
                        }
                        if (childFragmentManager.fragments.size > 0) {
                            carFragment = childFragmentManager.fragments[0] as CarControlFragment
                            fragment_container.viewTreeObserver.removeOnGlobalLayoutListener(this)
                            return
                        }
                        carFragment = CarControlFragment.newInstance(outRect)
                        val transaction = childFragmentManager.beginTransaction()
                        transaction.add(R.id.fragment_container, carFragment, CarControlFragment.KEY_CARCONTROL_FRAGMENT)
                        transaction.commitAllowingStateLoss()
                        fragment_container.viewTreeObserver.removeOnGlobalLayoutListener(this)

                    }
                })
    }


    /**
     * 下拉刷新结束时候，触发的view操作
     */
    private fun viewRefFinish() {
        view!!.post {
            if (pageSwitcher != null) {//用于规避Exception风险
                pageSwitcher!!.disableAllEvent(false)
                pageSwitcher!!.setRefreshing(false)
                MainPresenter.getInstance().notifyHomeRefreshing(false)
                view_loading.visibility = View.GONE
                view_loading.alpha = 1f
                Framework.getDataRecord().ctrlClicked("home", "refresh")
            }
        }
    }

    /**
     * 初始化地图点击事件等
     */
    private fun initMap(savedInstanceState: Bundle?) {
        mapView.onCreate(savedInstanceState)
        mAMap = mapView.map

        AMapUtil.setMapCustomStyleFile(context, mAMap)
        hideGaodeButton()

        val moveMapSubscribe = AMapUtilsZXQ.getCurrentLocation().doOnNext {
            if (it?.errorCode == 0) {
                val center = LatLng(it.latitude, it.longitude)
                mAMap.moveCamera(CameraUpdateFactory.changeLatLng(center))
            }
        }.subscribe()
        mCompositeDisposable.add(moveMapSubscribe)

        mAMap.setOnMapClickListener { verifyPinCode(true) }

        mAMap.setOnMapLoadedListener {
            LogUtils.d("onMapLoaded")
            isLoad = true
            iv_map_bg_alpha.visibility = View.GONE
            updateLocationOnMap()
        }
    }

}
