package org.zxq.teleri.orders.activity

import android.os.Bundle
import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.ProgressBar
import android.widget.RelativeLayout
import android.widget.TextView
import com.ebanma.sdk.core.net.exception.ApiException
import com.ebanma.sdk.flow.BMFlowManager
import com.ebanma.sdk.flow.bean.RestFlowInfoBean
import com.ebanma.sdk.flow.listener.OnRestFlowInfoListener
import kotlinx.android.synthetic.main.activity_rest_flow_info.*
import kotlinx.android.synthetic.main.headview_no_data.view.*
import kotlinx.android.synthetic.main.item_rest_flow_info.view.*
import org.zxq.teleri.R
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.ex.OnErrorResponse
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.orders.OrderConstants
import org.zxq.teleri.ui.base.SimpleBaseActivity
import org.zxq.teleri.ui.utils.DateUtils

/**
 *   @author     Deman.LU
 *   @date       2019/4/25
 *
 */
class RestFlowInfoActivity : SimpleBaseActivity() {

    private var dataType = 0
    private var flowPacketUsedDetailsRespDtos: MutableList<RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean> = mutableListOf()
    private lateinit var mRlNothing: RelativeLayout
    private lateinit var restFlowInfoAdapter: RestFlowInfoAdapter
    private var mRequestIdList = mutableListOf<Int>()


    companion object {
        const val TYPE_BY_FLOW = 1
        const val TYPE_BY_TIME = 2
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_rest_flow_info)
        setTitle("余量明细")
        dataType = intent.getIntExtra(OrderConstants.ORDER_DATA, 0)
        initViews()
        loadDataFlow()
    }

    override fun onDestroy() {
        super.onDestroy()
        if (mRequestIdList.size > 0) {
            BMFlowManager.bindService().cancelFlowRequestList(mRequestIdList)
            mRequestIdList.clear()
        }

    }

    private fun initViews() {
        mBack = findViewById(R.id.back)
        mBack.setOnClickListener {
            finish()
        }
        initHeadView()
        initListView()
    }

    private fun initListView() {
        pull_to_refresh_list_view.refreshableView.dividerHeight = 0
        flowPacketUsedDetailsRespDtos.clear()
        restFlowInfoAdapter = RestFlowInfoAdapter(flowPacketUsedDetailsRespDtos, this@RestFlowInfoActivity)
        pull_to_refresh_list_view.refreshableView.adapter = restFlowInfoAdapter
        pull_to_refresh_list_view.setOnRefreshListener { loadDataFlow() }
    }


    private fun initHeadView() {
        val headView = View.inflate(this, R.layout.headview_no_data, null)
        mRlNothing = headView.rl_nothing
        pull_to_refresh_list_view.refreshableView.addHeaderView(headView, null, false)
        mRlNothing.visibility = View.GONE
    }

    private fun loadDataFlow() {
        LogUtils.i("loadDataFlow")
        startRefreshLoading()
        val restFlowInfo = BMFlowManager.bindService().getRestFlowInfo(Framework.getAccountInject().vin,
                object : OnRestFlowInfoListener {
            override fun onSuccess(bean: RestFlowInfoBean) {
                pull_to_refresh_list_view.onRefreshComplete()
                when (dataType) {
                    OrderConstants.WIFI_TYPE -> {
                        refreshList(bean.resultObj?.wifi?.flowPacketUsedDetailsRespDtos)
                    }

                    OrderConstants.MUSIC_TYPE -> {
                        refreshList(bean.resultObj?.music?.flowPacketUsedDetailsRespDtos)
                    }

                    else -> {
                        LogUtils.w("$dataType is wrong type")
                    }
                }

            }

            override fun onFailed(apiException: ApiException) {
                pull_to_refresh_list_view.onRefreshComplete()
                LogUtils.i("ApiException:" + apiException.message)
                OnErrorResponse.getInstance().accept(apiException)
            }

        })
        mRequestIdList.add(restFlowInfo)
    }

    private fun refreshList(flowPacketUsedDetailsRespDtos: MutableList<RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean>?) {
        flowPacketUsedDetailsRespDtos?.let {
            this.flowPacketUsedDetailsRespDtos = it
            haveNothing(it.isEmpty())
            refreshListView(it)
        } ?: kotlin.run {
            this.flowPacketUsedDetailsRespDtos.clear()
            haveNothing(true)
            refreshListView(this.flowPacketUsedDetailsRespDtos)
        }

    }

    protected fun haveNothing(flag: Boolean) {
        mRlNothing.visibility = if (flag) View.VISIBLE else View.GONE
    }

    private fun refreshListView(flowPacketUsedDetailsRespDtos: List<RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean>) {
        restFlowInfoAdapter.updateData(flowPacketUsedDetailsRespDtos)
    }


    private fun startRefreshLoading() {
        pull_to_refresh_list_view.loadingLayoutProxy.setHeaderUpdateText("加载中...")
    }

    class RestFlowInfoAdapter(var flowPacketUsedDetailsRespDtos: List<RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean>,
                              var activity: RestFlowInfoActivity) : BaseAdapter() {

        override fun getCount(): Int = flowPacketUsedDetailsRespDtos.size

        override fun getItem(position: Int): RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean? {
            if (position >= 0 && position < flowPacketUsedDetailsRespDtos.size) {
                return flowPacketUsedDetailsRespDtos[position]
            }

            return null
        }

        override fun getItemId(position: Int): Long {
            return getItem(position)?.hashCode()?.toLong() ?: 0
        }

        /*call this must in ui thread*/
        fun updateData(flowPacketUsedDetailsRespDtos: List<RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean>) {
            this.flowPacketUsedDetailsRespDtos = flowPacketUsedDetailsRespDtos
            notifyDataSetChanged()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
            val view: View
            val mHolder: InfoViewHolder
            if (convertView == null) {
                mHolder = InfoViewHolder()
                view = View.inflate(activity, R.layout.item_rest_flow_info, null)
                mHolder.flow_title = view.flow_title
                mHolder.flow_progress = view.flow_progress
                mHolder.tv_description = view.tv_description
                mHolder.tv_last_day = view.tv_last_day
                view.tag = mHolder

            } else {
                view = convertView
                mHolder = view.tag as InfoViewHolder
            }

            bindHolder(mHolder, flowPacketUsedDetailsRespDtos[position], (flowPacketUsedDetailsRespDtos.size - 1) == position)
            return view
        }

        private fun bindHolder(mHolder: InfoViewHolder,
                               fPUDRBean: RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean,
                               isLast: Boolean) {
            mHolder.flow_title?.text = fPUDRBean.flowPacketName
            mHolder.flow_progress?.progress = calFlowProgress(fPUDRBean)
            val htmlString = "剩余流量：<font color=\"#333333\">" + fPUDRBean.visualizationRest + "</font>"+ "/${fPUDRBean.visualizationTotal}"
//            mHolder.tv_description?.text = "${fPUDRBean.visualizationRest}/${fPUDRBean.visualizationTotal}"
            mHolder.tv_description?.text = Html.fromHtml(htmlString)
            mHolder.tv_last_day?.text = calLastDay(fPUDRBean) + "失效"
        }

        private fun calLastDay(fPUDRBean: RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean): String {
            return DateUtils.getYYMMddHHmmFormat(fPUDRBean.expireDateTime)
        }

        private fun calFlowProgress(flowPacketUsedDetailsRespDtosBean: RestFlowInfoBean.ResultObjBean.FlowPacketUsedDetailsRespDtosBean): Int {
            var rest = 0
            when (flowPacketUsedDetailsRespDtosBean.flowType) {
                TYPE_BY_FLOW -> {
                    if(flowPacketUsedDetailsRespDtosBean.rest!=null &&
                            flowPacketUsedDetailsRespDtosBean.total!=null &&
                            flowPacketUsedDetailsRespDtosBean.total!=0f){

                        rest = (flowPacketUsedDetailsRespDtosBean.rest * 100L / flowPacketUsedDetailsRespDtosBean.total).toInt()
                    }else{
                        rest = 0
                    }
                }

                TYPE_BY_TIME -> {
                    if(flowPacketUsedDetailsRespDtosBean.totalTime!=0L){
                        rest = (flowPacketUsedDetailsRespDtosBean.restTime * 100L / flowPacketUsedDetailsRespDtosBean.totalTime).toInt()
                    }
                }
            }

            return rest
        }


        class InfoViewHolder {
            var flow_title: TextView? = null
            var flow_progress: ProgressBar? = null
            var tv_description: TextView? = null
            var tv_last_day: TextView? = null
        }
    }
}