package org.zxq.teleri.orders.fragment

import android.annotation.SuppressLint
import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.Paint
import android.os.Bundle
import android.provider.Settings
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.BaseAdapter
import android.widget.LinearLayout
import android.widget.TextView
import com.ebanma.sdk.core.net.exception.ApiException
import com.ebanma.sdk.flow.BMFlowManager
import com.ebanma.sdk.flow.bean.GetGoodsListInfoBean
import com.ebanma.sdk.flow.bean.GoodsBean
import com.ebanma.sdk.flow.bean.GoodsSpecBean
import com.ebanma.sdk.flow.bean.RestFlowInfoBean
import com.ebanma.sdk.flow.listener.OnGoodListsInfoInfoListener
import com.ebanma.sdk.flow.service.IFlowService
import kotlinx.android.synthetic.main.fragment_music_entertainment.view.*
import org.zxq.teleri.R
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.ex.OnErrorResponse
import org.zxq.teleri.core.net.exception.ErrorResponseException
import org.zxq.teleri.core.utils.Json
import org.zxq.teleri.core.utils.LogUtils
import org.zxq.teleri.core.utils.NetUtil
import org.zxq.teleri.core.utils.ParseUtil
import org.zxq.teleri.global.MagicConstant
import org.zxq.teleri.model.request.orders.MusicOrderCreateRequest
import org.zxq.teleri.orders.OrderConstants
import org.zxq.teleri.orders.activity.DataFlowActivity
import org.zxq.teleri.orders.activity.PersonalCenterConfirmOrderActivity
import org.zxq.teleri.orders.activity.RestFlowInfoActivity
import org.zxq.teleri.orders.bean.OrderCreateData
import org.zxq.teleri.personalcenter.bean.FlowDataBean
import org.zxq.teleri.ui.base.SimpleBaseFragment
import org.zxq.teleri.ui.utils.UIUtils
import java.text.DecimalFormat
import java.util.*

/**
 *   @author     Deman.LU
 *   @date       2019/4/24
 *
 */
abstract class FlowFragment : SimpleBaseFragment() {

    /*GoodsBean 是一个数组，但是实际只返回一个*/
    protected lateinit var currentGoodsBean: GoodsBean
    /*所有可选流量的列表，100M，300M*/
    protected lateinit var goodsSpec: List<GoodsSpecBean>
    protected lateinit var selectSpec: GoodsSpecBean
    protected lateinit var rootView: View
    protected lateinit var btn_set_network: View
    protected lateinit var btn_reload: View
    protected var rmbDiscountPrice: String? = ""
    protected var rmbPrice: String? = ""
    protected var zxqPrice: String? = ""
    protected var mRequestIdList = mutableListOf<Int>()


    abstract fun getMusicIconRes(): Int

    abstract fun getCatId(): IFlowService.GoodsCat

    abstract fun getTitle(): String

    abstract fun getSurPlusDataFlow(data: FlowDataBean): FlowDataBean.FlowSurplusBean?

    abstract fun getDataRecordPageName(): String

    abstract fun getDataFlowType(): Int

    companion object {
        const val REQUEST_NET_CODE = 0x18
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        rootView = initView(inflater, container)
        initData(rootView)
        setPageName(getDataRecordPageName())
        resetDataFlow()
        return rootView
    }

    override fun onResume() {
        super.onResume()
//        checkNetwork()
        reload()
    }

    override fun onDestroy() {
        super.onDestroy()
        if (mRequestIdList.size>0){
            BMFlowManager.bindService().cancelFlowRequestList(mRequestIdList)
            mRequestIdList.clear()
        }

    }


    private fun initData(rootView: View) {
        rootView.btn_buy_exchange.setOnClickListener {
            buyExchange()
        }
        btn_set_network = rootView.findViewById<View>(R.id.btn_set_network)
        btn_set_network.setOnClickListener {
            setNetWork(REQUEST_NET_CODE)
        }
        btn_reload = rootView.findViewById(R.id.btn_reload)
        btn_reload.setOnClickListener {
            reload()
        }

        rootView.gv_dataflow.setOnItemClickListener { adapterView, view, i, l ->
            Framework.getDataRecord().ctrlClicked("person_account", "acc_traffic")
            activity?.let { context ->
                val dataflowAdapter = DataflowAdapter(context, goodsSpec, i)
                rootView.gv_dataflow.adapter = dataflowAdapter
                selectSpec = goodsSpec[i]
                setPrice(goodsSpec[i])
                setDespaiction(goodsSpec[i])
                rootView.tv_scope.text = goodsSpec[i].scope
            }
        }

        rootView.tv_query_margin.setOnClickListener {
            val intent = Intent(activity, RestFlowInfoActivity::class.java)
            intent.putExtra(OrderConstants.ORDER_DATA, getDataFlowType())
            startActivity(intent)
        }
    }

    private fun initView(inflater: LayoutInflater, container: ViewGroup?): View {
        val view = inflater.inflate(R.layout.fragment_music_entertainment,
                container, false)
        view.imv_music.setBackgroundResource(getMusicIconRes())
        view.imv_music.visibility = View.VISIBLE

        view.tv_music_flow.text = getTitle()
        view.tv_music_flow.visibility = View.VISIBLE

        return view

    }

    private fun buyExchange() {
        val isNetConnected = NetUtil.isNetWorkConnected()
        if (!isNetConnected) {
            UIUtils.shortToast(R.string.network_disconnected)
            return
        }

        if (!(rootView.btn_buy_exchange.tag as Boolean)) {
            UIUtils.shortToast(R.string.no_data_quantity)
            return;
        }

        orderCreate()
    }


    private fun setNetWork(requestCode: Int) {
        var intent: Intent?
        intent = Intent(Settings.ACTION_SETTINGS)
        startActivityForResult(intent, requestCode)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == DataFlowActivity.RESULT_TO_PAY || resultCode == PersonalCenterConfirmOrderActivity.RESULT_TO_CAR_WALLET && "isSuccess" == data!!.getStringExtra("isSuccess")) {
            activity!!.finish()
        }
//        if (requestCode == REQUEST_NET_CODE) {
//            if (NetUtil.isNetWorkConnected()) {
//                checkNetwork()
//            }
//        }
    }

    private fun checkNetwork() {
        if (NetUtil.isNetWorkConnected()) {
//            loadFlowData()
            getGoodsList()
            rootView.view_no_net.visibility = View.GONE
        } else {
            rootView.view_no_net.visibility = View.VISIBLE
        }
    }

    private fun reload() {
        checkNetwork()
        if (NetUtil.isNetWorkConnected()) {
            if (activity is DataFlowActivity) {
                (activity as DataFlowActivity).loadFlowData();
            }
        }
    }

    private fun parserPrice(price: String?): HashMap<String, String>? {
        var goodsMap: HashMap<String, String>? = null
        //        String price = goodsSpecBean.getPrice();
        if (price != null && "" != price) {
            goodsMap = HashMap()
            if (price.contains("|")) {
                val strings = price.split("\\|".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                for (string in strings) {
                    val split = string.split(":".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                    goodsMap[split[0]] = split[1]
                }
            } else {
                val split = price.split(":".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                goodsMap[split[0]] = split[1]
            }
            return goodsMap
        }
        return goodsMap
    }

    @SuppressLint("SetTextI18n")
    private fun getGoodsList() {
        val goodsCatId = getCatId()
        showLoadingDialog()
        val requestId = BMFlowManager.bindService().getGoodLists(goodsCatId, Framework.getAccountInject().vin, object : OnGoodListsInfoInfoListener {
            override fun onSuccess(goodsListData: GetGoodsListInfoBean) {
                closeLoadingDialog()
                rootView.btn_buy_exchange.tag = true
                if (goodsListData != null) {
                    val goodsList = goodsListData.goods
                    currentGoodsBean = goodsList[0]
                    goodsSpec = currentGoodsBean.goodsSpec
                    selectSpec = goodsSpec[0]
                    rootView.tv_product_presentation.text = resources.getString(R.string.product_presentation) +
                            currentGoodsBean.description

                    setDespaiction(selectSpec)
                    setNetData(0, selectSpec)
                    setPrice(selectSpec)
                } else {
                    rootView.btn_buy_exchange.tag = false
                }
            }

            override fun onFailed(apiException: ApiException) {
                closeLoadingDialog()
                rootView.btn_buy_exchange.tag = false
                OnErrorResponse.getInstance().accept(apiException)
            }

        })
        mRequestIdList.add(requestId)
    }


    private fun orderCreate() {
        showLoadingDialog()
        rootView.btn_buy_exchange.isEnabled = false
        val musicOrderCreateRequest = MusicOrderCreateRequest(selectSpec.gsId, currentGoodsBean.gversion)
        musicOrderCreateRequest.subscribe({ s ->
            closeLoadingDialog()
            rootView.btn_buy_exchange.isEnabled = true
            try {
                val orderCreateData = Json.from(s, OrderCreateData::class.java)
                if (orderCreateData != null) {
                    val payOrderId = orderCreateData.pay_order_id
                    val uoId = orderCreateData.uoId
                    val intent = Intent(UIUtils.getContext(),
                            PersonalCenterConfirmOrderActivity::class.java)
                    intent.putExtra(MagicConstant.PAY_ORDER_ID, payOrderId)
                    intent.putExtra(MagicConstant.UO_ID, uoId)
                    intent.putExtra(MagicConstant.GOODS_CAT_ID, getCatId().value)
                    intent.putExtra(MagicConstant.GOODS_FLOW_ITEM,selectSpec)
                    startActivityForResult(intent,
                            DataFlowActivity.RESULT_TO_PAY)
                }
            } catch (exp: Exception) {
                exp.printStackTrace()
                LogUtils.w(exp.toString())
            }
        }, { throwable ->
            rootView.btn_buy_exchange.isEnabled = true
            closeLoadingDialog()
            if (throwable is ErrorResponseException) {
                OnErrorResponse.getInstance().accept(throwable)
            }
        })
    }

    private fun setDespaiction(goodsSpecBean: GoodsSpecBean) {
        val specification = goodsSpecBean.specification
        var dataFlowString = ""
        if (specification.size == 2) {
            dataFlowString = specification[0].spec_value + "：" + specification[1].spec_value
        }
        rootView.tv_dataflow_description.text = dataFlowString

    }

    private fun setNetData(position: Int, selectSpec: GoodsSpecBean) {
        val mDataFlowAdapter = DataflowAdapter(activity!!, goodsSpec, position)
        rootView.gv_dataflow.adapter = mDataFlowAdapter
        rootView.tv_scope.text = selectSpec.scope
    }


    private fun setPrice(goodsSpecBean: GoodsSpecBean) {
        val discountprice = goodsSpecBean.discountprice
        val goodsPrice = parserPrice(goodsSpecBean.price)
        var discountPrice: HashMap<String, String>? = null
        val keySet = goodsPrice!!.keys
        if (null != discountprice && !TextUtils.isEmpty(discountprice)) {
            //解析优惠价
            rootView.tv_discountprice_number_sign.visibility = View.VISIBLE
            rootView.tv_discountprice_number.visibility = View.VISIBLE
            rootView.tv_price_number.textSize = 12f
            rootView.tv_price_number.setTextColor(Color.parseColor("#999999"))
            rootView.tv_price_number.paint.flags = Paint.STRIKE_THRU_TEXT_FLAG
            rootView.tv_price_number_sign.textSize = 12f
            rootView.tv_price_number_sign.setTextColor(Color.parseColor("#999999"))
            rootView.tv_price_number_sign.paint.flags = Paint.STRIKE_THRU_TEXT_FLAG

            discountPrice = parserPrice(goodsSpecBean.discountprice)
        } else {
            rootView.tv_discountprice_number_sign.visibility = View.GONE
            rootView.tv_discountprice_number.visibility = View.GONE
            rootView.tv_price_number_sign.textSize = 15f
            rootView.tv_price_number_sign.setTextColor(Color.parseColor("#ff333333"))
            rootView.tv_price_number_sign.paint.flags = Paint.ANTI_ALIAS_FLAG
            rootView.tv_price_number.setTextColor(Color.parseColor("#ff333333"))
            rootView.tv_price_number.textSize = 15f
            rootView.tv_price_number.paint.flags = Paint.ANTI_ALIAS_FLAG
        }

        if (keySet.contains("11") && keySet.contains("10")) {
            rootView.tv_price_number.setVisibility(View.VISIBLE)
//            rootView.tv_price_number_sign.setVisibility(View.VISIBLE)
            rootView.tv_price_number_sign.setVisibility(View.GONE)
            rootView.tv_score.setVisibility(View.VISIBLE)
            rootView.tv_or.setVisibility(View.VISIBLE)
            rootView.tv_total_count.setVisibility(View.VISIBLE)

            if (null != discountPrice) {
                //优惠价
                rmbDiscountPrice = discountPrice["11"]
                rootView.tv_discountprice_number.setText(rmbDiscountPrice)
            } else {
                rmbDiscountPrice = ""
            }
            //原价
            rmbPrice = goodsPrice["11"]
            rootView.tv_price_number.setText(rmbPrice+"元")
            zxqPrice = goodsPrice["10"]
            rootView.tv_score.setText(zxqPrice)
        } else if (keySet.contains("11") && !keySet.contains("10")) {
            rootView.tv_price_number.setVisibility(View.VISIBLE)
//            rootView.tv_price_number_sign.setVisibility(View.VISIBLE)
            rootView.tv_price_number_sign.setVisibility(View.GONE)
            rootView.tv_score.setVisibility(View.GONE)
            rootView.tv_or.setVisibility(View.GONE)
            rootView.tv_total_count.setVisibility(View.GONE)

            if (null != discountPrice) {
                //优惠价
                rmbDiscountPrice = discountPrice["11"]
                rootView.tv_discountprice_number.setText(rmbDiscountPrice)
                //                rootView.tv_price_number_sign.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG );
                //                rootView.tv_price_number.getPaint().setFlags(Paint.STRIKE_THRU_TEXT_FLAG );
            } else {
                rmbDiscountPrice = ""
            }
            //原价
            rmbPrice = goodsPrice["11"]
            //                rootView.tv_price_number_sign.setTextColor(Color.parseColor("#ff3400"));
            //                rootView.tv_price_number.setTextColor(Color.parseColor("#ff3400"));
            rootView.tv_price_number.setText(rmbPrice+"元")
            zxqPrice = ""
        } else if (!keySet.contains("11") && keySet.contains("10")) {
            rootView.tv_price_number.setVisibility(View.GONE)
            rootView.tv_price_number_sign.setVisibility(View.GONE)
            rootView.tv_score.setVisibility(View.VISIBLE)
            rootView.tv_or.setVisibility(View.GONE)
            rootView.tv_total_count.setVisibility(View.VISIBLE)

            rmbPrice = ""
            zxqPrice = goodsPrice["10"]
            rootView.tv_score.setText(zxqPrice)
        } else {
            rootView.tv_price_number.setVisibility(View.GONE)
            rootView.tv_price_number_sign.setVisibility(View.GONE)
            rootView.tv_score.setVisibility(View.VISIBLE)
            rootView.tv_or.setVisibility(View.GONE)
            rootView.tv_total_count.setVisibility(View.VISIBLE)

            rmbPrice = ""
            zxqPrice = ""
        }
    }


    @SuppressLint("SetTextI18n")
    private fun loadFlowData() {
        LogUtils.i("loadFlowData")
        if (TextUtils.isEmpty(Framework.getAccountInject().vin)) {
            LogUtils.w("loadFlowData vin is empty")
            return
        }
        /* val disposable = SummaryRequest().subscribe({ s ->
             try {
                 val data = Json.from(s, FlowDataBean::class.java)
                 if (data != null) {
                     val flowSurp = getSurPlusDataFlow(data)
                     flowSurp?.let {
                         val text = parseSurplusBean(flowSurp)
                         rootView.tv_left_data.text = resources.getString(R.string.default_left_flow1)+text
                         setProgress(flowSurp)
                     }?:kotlin.run{
                         resetDataFlow()
                     }
                 }
             } catch (e: Exception) {
                 e.printStackTrace()
             }
         }, {
             //                OnErrorResponse.getInstance().accept(throwable);
         })
         mCompositeDisposable.add(disposable)*/
    }


    protected abstract fun parseFlowData(result: RestFlowInfoBean.ResultObjBean): Pair<Int, String>;

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        getFlowLiveData();
    }

    /**
     * 获取流量数据
     */
    private fun getFlowLiveData() {
        if (activity is DataFlowActivity) {
            (activity as DataFlowActivity).flowLiveData.observe(this, Observer<RestFlowInfoBean.ResultObjBean> {
                it?.let {
                    parseFlowData(it)?.apply {
                        rootView.tv_left_data.text = resources.getString(R.string.default_left_flow1) + if (TextUtils.equals("0M", second)) "- - " else second
                        rootView.flow_progress.progress = first
                    }
                } ?: kotlin.run { resetDataFlow() }
            });
        }
    }


    private fun resetDataFlow() {
        rootView.tv_left_data.text = resources.getString(R.string.default_left_flow)
        rootView.flow_progress.progress = 0
    }

    private fun setProgress(flowSurp: FlowDataBean.FlowSurplusBean) {
        try {
            var current = 0
            current = (flowSurp.surplus.toDouble() * 100 / flowSurp.flow_original.toDouble()).toInt()
            rootView.flow_progress.progress = current
        } catch (e: Exception) {
        }
    }

    private fun parseSurplusBean(flowSurp: FlowDataBean.FlowSurplusBean): String {
        return when {
            flowSurp.flow_type.isNullOrEmpty() || "1" == flowSurp.flow_type -> when {
                (ParseUtil.parseDouble(flowSurp.surplus) >= 1024) && (flowSurp.unit == "M") -> formatDouble(flowSurp.surplus) + "G"
                else -> flowSurp.surplus + flowSurp.unit
            }
            else -> flowSurp.surplus + flowSurp.unit
        }
    }

    private fun formatDouble(dstr: String): String {
        if (TextUtils.isEmpty(dstr)) {
            return ""
        }
        var str = ""
        try {
            var d = java.lang.Double.parseDouble(dstr)
            if (d >= 1024) {
                d = d / 1024
            }
            str = String.format(Locale.CANADA, "%.2f", d)
        } catch (e: NumberFormatException) {
            e.printStackTrace()
        }

        return str
    }

    internal class ViewHolder {
        var tvNum: TextView? = null
        var tvPrice: TextView? = null
        var ll: LinearLayout? = null

    }

    internal inner class DataflowAdapter(private val context: Context, private val list: List<GoodsSpecBean>,
                                         private val position: Int) : BaseAdapter() {

        override fun getCount(): Int {
            return list.size
        }

        override fun getItem(i: Int): Any {
            return list[i]
        }

        override fun getItemId(i: Int): Long {
            return i.toLong()
        }

        override fun getView(i: Int, view: View?, viewGroup: ViewGroup): View {
            var view = view
            val df = DecimalFormat("#0.00")
            var viewHolder: ViewHolder? = null
            if (view == null) {
                viewHolder = ViewHolder()
                view = LayoutInflater.from(context).inflate(R.layout.item_gv_dataflow, null)
                viewHolder.tvNum = view!!.findViewById<View>(R.id.tv_data_flow_num) as TextView
                viewHolder.ll = view.findViewById<View>(R.id.ll_data_flow_num) as LinearLayout
                viewHolder.tvPrice = view.findViewById<View>(R.id.tv_data_Price) as TextView
                view.tag = viewHolder
            } else {
                viewHolder = view.tag as ViewHolder
            }
            viewHolder.tvNum!!.text = list[i].specification[0].spec_value + ""
            if (list[i].discountprice != null && !TextUtils.isEmpty(list[i].discountprice)) {
                val discountprice = parserPrice(list[i].discountprice)
                viewHolder.tvPrice!!.text = "" + df.format(java.lang.Double.parseDouble(discountprice!!.get("11"))) + resources.getString(R.string.rmb_yuan)
            } else if (list[i].price != null && !TextUtils.isEmpty(list[i].price)) {
                val price = parserPrice(list[i].price)
                viewHolder.tvPrice!!.text = "" + df.format(java.lang.Double.parseDouble(price!!.get("11"))) + resources.getString(R.string.rmb_yuan)
            }
            if (position == i) {
                viewHolder.ll!!.background = resources.getDrawable(R.drawable.person_center_bg_liuliang_press)
                viewHolder.tvNum!!.setTextColor(Color.parseColor("#FF13D9C9"))
                viewHolder.tvPrice!!.setTextColor(Color.parseColor("#FF13D9C9"))
            } else {
                viewHolder.ll!!.background = resources.getDrawable(R.drawable.person_center_bg_liuliang_nor)
                viewHolder.tvNum!!.setTextColor(Color.parseColor("#FF666666"))
                viewHolder.tvPrice!!.setTextColor(Color.parseColor("#FF666666"))
            }
            return view
        }
    }
}