package org.zxq.teleri.windvareweb.plugin

import android.content.ActivityNotFoundException
import android.content.Intent
import android.location.LocationManager
import android.provider.Settings
import android.taobao.windvane.jsbridge.WVApiPlugin
import android.taobao.windvane.jsbridge.WVCallBackContext
import android.taobao.windvane.jsbridge.WVResult
import com.amap.api.location.AMapLocation
import com.amap.api.location.AMapLocationClient
import org.zxq.teleri.account.datamodel.UserLogin
import org.zxq.teleri.core.utils.LogUtils
import com.ebanma.sdk.core.domain.DomainUtil
import org.zxq.teleri.ui.utils.UserRelationManager
import org.zxq.teleri.utils.AMapLocationManager

/**
 *   @author     Deman.LU
 *   @Date       2019/1/14
 *
 */
class BMMaintainJSBridge:WVApiPlugin() {
    private var mLocationManager: LocationManager? = null
    private var mLocationClient: AMapLocationClient? = null

    override fun execute(action: String?, params: String?, callback: WVCallBackContext?): Boolean {
        return action?.let {
            when(it){
                "toGPSSettingPage"->{
                    toGPSSettingPage(callback)
                    true
                }
                "getUserRelation"->{
                    getUserRelation(callback)
                    true
                }
                "getNativeData"->{
                    getNativeData(callback)
                    true
                }
                else->{
                    false
                }
            }
        }?:kotlin.run {
            false
        }

    }


    private fun getNativeData(callback: WVCallBackContext?) {
        getCity(callback)
    }

    private fun getUserRelation(callback: WVCallBackContext?) {
        val result = WVResult()
        result.addData("relation",UserRelationManager.getRelation())
        callback?.success(result)
    }

    private fun toGPSSettingPage(callback: WVCallBackContext?) {

        val intent = Intent()
        intent.action = Settings.ACTION_LOCATION_SOURCE_SETTINGS
        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        try {
            mContext.startActivity(intent)
        } catch (ex: ActivityNotFoundException) {
            intent.action = Settings.ACTION_SETTINGS// General settings activity
            try {
                mContext.startActivity(intent)
            } catch (e: Exception) {
                e.printStackTrace()
            }

        }
        callback?.success()

    }

    private fun getCity(callback: WVCallBackContext?){
        LogUtils.i("getCity")
        AMapLocationManager.instance.getLocationASync(object:AMapLocationManager.OnAMapLocationListener{
            override fun onAMapLocation(aMapLocation: AMapLocation) {
                callCityResult(aMapLocation,callback)
            }
        })
    }

    private fun callCityResult(aMapLocation: AMapLocation, callback: WVCallBackContext?) {
        val result= WVResult()
        result.addData("url_info_list", DomainUtil.BizType.`as`.domainUrlWithoutVersion)
        result.addData("province", aMapLocation.province)
        result.addData("city", aMapLocation.city)
        result.addData("lat", aMapLocation.latitude)
        result.addData("lng", aMapLocation.longitude)
        result.addData("url_my_photo", UserLogin.getAccountA().photoUrl)
        LogUtils.i("callCityResult")
        callback?.success(result)
    }
}