package org.zxq.teleri.windvareweb.plugin

import android.app.Activity
import android.content.Context
import android.location.LocationManager
import android.os.Handler
import android.support.v4.app.FragmentActivity
import android.taobao.windvane.jsbridge.WVApiPlugin
import android.taobao.windvane.jsbridge.WVCallBackContext
import android.taobao.windvane.jsbridge.WVResult
import android.taobao.windvane.webview.IWVWebView
import android.util.ArrayMap
import com.alibaba.android.arouter.launcher.ARouter
import com.alibaba.fastjson.JSONObject
import com.amap.api.services.core.LatLonPoint
import com.ebanma.sdk.core.constant.Platform
import com.ebanma.sdk.core.net.openapi.SignUtils
import org.json.JSONException
import org.zxq.teleri.R
import org.zxq.teleri.arouter.constant.ARouterUrl
import org.zxq.teleri.bean.LatLonPointBean
import org.zxq.teleri.common.tools.LoggerUtils
import org.zxq.teleri.compoentservice.pay.IPayProvider
import org.zxq.teleri.compoentservice.pay.OnPayResultListener
import org.zxq.teleri.core.AppConfig
import org.zxq.teleri.core.Framework
import org.zxq.teleri.core.utils.*
import org.zxq.teleri.dialog.ContactPhonePopupWindow
import org.zxq.teleri.ui.model.style.Button
import org.zxq.teleri.ui.theme.ThemeManager
import org.zxq.teleri.ui.utils.AppUtils
import org.zxq.teleri.ui.utils.DisplayUtil
import org.zxq.teleri.ui.utils.UIUtils
import org.zxq.teleri.utils.BaseJsCallBackUtils
import org.zxq.teleri.utils.VehiclePositionUtils
import java.text.SimpleDateFormat
import java.util.*

/**
 *   @author     Deman.LU
 *   @Date       2019/1/11
 *
 */
class BmJsBridgePlugin:WVApiPlugin() {

    companion object {
        private const val PAY_RESULT_SUCCESS = 1//成功
        private const val PAY_RESULT_FAILED = 2//失败
        private const val PAY_RESULT_CANCELED = 3//取消
        private const val PAY_RESULT_UNKNOWN = 4//未知
        private const val PAY_RESULT_UNCONFIRMED = 5//未确认，目前未收过到此回调
    }


    override fun initialize(context: Context?, webView: IWVWebView?) {
        super.initialize(context, webView)
        LogUtils.i("init here:$webView")
    }

    /**
     * js 的规则，
     */
    override fun execute(action: String?, params: String?, callback: WVCallBackContext?): Boolean {
        LogUtils.i("WV BmJsBridgePlugin $action")
        val map = BmJsonHelper.getMap(params)
        return action?.let {
            when(it){
                "toAlipay"->{
                    val url = map?.get("url") as? String
                    val payOrderId = map?.get("payOrderId") as? String
                    toAlipay(url,payOrderId,callback)
                    true
                }
                "errorMessageHandler"->{
                    errorMessageHandler(map,callback)
                    true
                }
                "messageToastAction"->{
                    val toast = map?.get("msg") as? String
                    messageToastAction(toast,callback)
                    true
                }
                "getNativeData"->{
                    getNativeData(callback)
                    true
                }
                "getButtonPrimaryColor"->{
                    getButtonPrimaryColor(callback)
                    true
                }
                "getOemToken"->{
                    val oemCode = map?.get("oem") as? String
                    getOemToken(oemCode,callback)
                    true
                }
                "callTel"->{
                    callTel(map,callback)
                    true
                }
                "statAnalysis"->{
                    statAnalysis(params,callback)
                    true
                }
                "startToNav"->{
                    startToNav(params,callback)
                    true
                }
                "startPaySet"->{
                    startPaySet(callback)
                    true
                }
                "finish"->{
                    finish(callback)
                    true
                }
                "reloadPage"->{
                    reloadPage(callback)
                    true
                }
                "isGPSOpen"->{
                    isGPSOpen(callback)
                    true
                }
                "getSign"->{
                    getSignMethod(params,callback)
                    true
                }
                "isNetWorkConnected"->{
                    isNetWorkConnected(callback)
                    true
                }
                else->{
                    false
                }
            }

        }?:kotlin.run {
            false
        }
    }

    private fun isNetWorkConnected(callback: WVCallBackContext?) {
        val flag = NetUtil.isNetWorkConnected()
        var result = WVResult()
        result.addData("connected",flag)
        callback?.success(result)
    }

    private fun getSignMethod(params: String?, callback: WVCallBackContext?) {

        params?.let {
            val signParams = ArrayMap<String, String>()
            signParams["appKey"] = AppConfig.ZXQ_KEY
            signParams["timestamp"] = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA).format(Date())
            try {
                val jsonObject = org.json.JSONObject(params)
                val keys = jsonObject.keys()
                while (keys.hasNext()) {
                    val key = keys.next()
                    val value = jsonObject.getString(key)
                    signParams[key] = value
                }
            } catch (e: JSONException) {
                e.printStackTrace()
            }

            val sign = SignUtils.signByMd5(signParams, AppConfig.ZXQ_SECRET)
            val result = WVResult()
            result.addData("sign",sign)
            callback?.success(result)
        }?:kotlin.run {
            callback?.error()
        }

    }

    private fun isGPSOpen(callback: WVCallBackContext?) {
        val result = WVResult()
        val mLocationManager = mContext?.getSystemService(Context.LOCATION_SERVICE) as? LocationManager
        mLocationManager?.let {
            result.addData("status",it.isProviderEnabled(LocationManager.GPS_PROVIDER))
        }?:kotlin.run {
            result.addData("status",false)
        }
        callback?.success(result)
    }

    private fun startPaySet(callback: WVCallBackContext?) {
        ARouter.getInstance().build(ARouterUrl.PAY_SETTING_ACTIVITY).navigation()
        callback?.success()
    }

    private fun reloadPage(callback: WVCallBackContext?) {
        mWebView?.refresh()
        callback?.success()
    }


    /**
     * h5埋点
     */
    private fun statAnalysis(params: String?, callback: WVCallBackContext?) {
        params?.let {
            WebDataUtils.statAnalysis(getMap(params))
            callback?.success()
        }?:kotlin.run {
            callback?.error()
        }

    }

    private fun getMap(json: String): Map<String, Any> {
        val jsonObject = JSONObject.parseObject(json)
        val valueMap = HashMap<String, Any>()
        valueMap.putAll(jsonObject)
        return valueMap
    }

    /**
     * 开启导航
     */
    private fun startToNav(params: String?, callback: WVCallBackContext?) {
        params?.let {
            try {
                val navData = Json.from(params,NavData::class.java)
                with(navData){

                    if(!startLat.isNullOrEmpty() && !startLng.isNullOrEmpty()
                        && !endLat.isNullOrEmpty() && !endLng.isNullOrEmpty()){
                        val planType = if ("walk".equals(navMode, ignoreCase = true)) 1 else 2
                        val startBean = LatLonPointBean()
                        startBean.setmPoint(LatLonPoint(startLat!!.toDouble(), startLng!!.toDouble()))
                        startBean.setmAddress("当前位置")
                        val endBean = LatLonPointBean()
                        endBean.setmPoint(LatLonPoint(endLat!!.toDouble(), endLng!!.toDouble()))
                        endBean.setmAddress("门店位置")
                        VehiclePositionUtils.getInstance().toNavigate(mContext as FragmentActivity,
                                false, planType, startBean, endBean)
                    }else{
                        UIUtils.shortToast(R.string.current_location_error)
                    }
                }

                callback?.success()
            } catch (e: Exception) {
                callback?.error()
            }
        }?:kotlin.run {
            callback?.error()
        }

    }

    /**
     * 打电话
     */
    private fun callTel(map: Map<String, Any>?, callback: WVCallBackContext?) {
        map?.let {
            LoggerUtils.d("ZXQ", "tel:$map")
            val tel = map["tel"] as? String
            if (mContext != null && mWebView != null) {
                Handler(ContextPool.peek().mainLooper).post {
                    ContactPhonePopupWindow.show(mContext as Activity, mWebView.view)
                            .setTelephoneNumbers(tel)
                }
            }
            callback?.success()
        }?:kotlin.run {
            callback?.error()
        }
    }

    /**
     * 获取对应OEM的token如返回""，说明APP本地没登录过该车厂账号，走登录流程，登录成功再调JavaScript:loginSuccess
     */
    private fun getOemToken(oemCode: String?, callback: WVCallBackContext?) {
        oemCode?.let {
            val oemToken = Framework.getTempAccount(oemCode).token
            val result = WVResult()
            result.addData("oemToken",oemToken)
            callback?.success(result)
        }?:kotlin.run {
            callback?.error()
        }

    }

    private fun getButtonPrimaryColor(callback: WVCallBackContext?) {
        val style = ThemeManager.getTheme().getStyle<Button>("Button@common")
        val result= WVResult()
        if (style == null) {
            result.addData("color", "#FF13D9C9")
        } else {
            result.addData("color", style.bg_normal)
        }
        callback?.success(result)
    }


    private fun getNativeData(callback: WVCallBackContext?) {

        val result= WVResult()
        result.addData("tokenA", Framework.getAccount(Platform.A).token)
        result.addData("token", Framework.getAccount(Platform.B).token)
        result.addData("oemCode", Framework.getAccountInject().oemCode)
        result.addData("vin", Framework.getAccountInject().vin)
        result.addData("headUrl", Framework.getAccount(Platform.A).photoUrl)
        result.addData("statusBarHeight", DisplayUtil.px2dip(DisplayUtil.getStatusBarHeight().toDouble()))
        result.addData("creditCode", Framework.getSecurityBox().makeCreditCode())
        result.addData("secureTimestamp", Framework.getSecurityBox().securityTimeStamp)
        result.addData("banmaId", Framework.getAccount(Platform.A).id)
        result.addData("terminalType", "2")
        result.addData("clientId", Framework.getSecurityBox().mobileUuid)
        result.addData("randomValue", Framework.getSecurityBox().securityRandomValue)
        result.addData("oemMobile", Framework.getAccount(Platform.B).mobile)
        LogUtils.i("getNativeData result $callback")
        callback?.success(result)
    }

    private fun messageToastAction(params: String?, callback: WVCallBackContext?) {
        AppUtils.showToast(params)
        callback?.success()
    }

    private fun errorMessageHandler(map: Map<String, Any>?, callback: WVCallBackContext?) {
        if (mContext == null) {
            return
        }

        map?.let {
            try {
                val type = map["type"] as? Int
                val data = map["data"] as? org.json.JSONObject
                val vin  = map["vin"] as? String
                    LogUtils.d("errorHandler type$type--content${data.toString()}--vin=$vin")
                    if (data.toString().isEmpty()) {
                        callback?.error()
                        return
                    }
                    BaseJsCallBackUtils.requestErrorHandlerH5(type?:0, data.toString(), vin)
                callback?.success()
            } catch (e: Exception) {
                callback?.error()
            }
        }?:kotlin.run {
            callback?.error()
        }

    }

    private fun toAlipay(params: String?,payOrderId:String?, callback: WVCallBackContext?) {
        params?.let {
            nativeToAlipay(it,payOrderId,callback)
        }?:kotlin.run {
            callback?.error()
        }
    }

    /*
    * 需要多次check，如果context有问题，需要进行新的开发
    * */
    private fun nativeToAlipay(url: String,payOrderId:String?,callback: WVCallBackContext?) {
        val iPayProvider = ARouter.getInstance().build(ARouterUrl.URL_PAY_MANAGER).navigation() as IPayProvider
        iPayProvider.pay(mContext as Activity,
                payOrderId.toString(),
                url,
                object : OnPayResultListener {
                    override fun onPaySucceed() {
                        callJsAliPayResult(PAY_RESULT_SUCCESS)
                    }

                    override fun onPayFailed() {
                        callJsAliPayResult(PAY_RESULT_FAILED)
                    }

                    override fun onPayUnconfirmed() {
                        callJsAliPayResult(PAY_RESULT_UNCONFIRMED)
                    }

                    override fun onPayCancled() {
                        callJsAliPayResult(PAY_RESULT_CANCELED)
                    }

                    override fun onPayUnknow() {
                        callJsAliPayResult(PAY_RESULT_UNKNOWN)
                    }

                })
    }

    private fun callJsAliPayResult(ret: Int) {
        if (mContext == null) {
            return
        }
        LogUtils.i("ret: $ret")
        ThreadUtils.runOnMainThread{
            mWebView.loadUrl("javascript:onAlipayResult('$ret')")
        }
    }

    private fun finish(callback: WVCallBackContext?) {
        mContext?.let {
            if(it is Activity){
                it.finish()
                callback?.success()
                return
            }
        }
        callback?.error()
    }
}
